"use strict";
var chai_1 = require('chai');
var main_1 = require('./main');
var compiler = main_1.buildCompiler('en');
describe('buildCompiler', function () {
    it('should fail with invalid locale', function () {
        chai_1.expect(function () { return main_1.buildCompiler('blahblah'); }).to.throw();
    });
    it('should build compiler with valid constant compiler function', function () {
        var staticCompile = main_1.buildCompiler('en').constant;
        var formatter = staticCompile('hello');
        chai_1.expect(formatter()).to.be.equals('hello');
    });
    it('should build compiler with valid callback compiler function', function () {
        var callbackCompile = main_1.buildCompiler('en').callback;
        var formatter = callbackCompile(function (human) { return ("hello " + human); });
        chai_1.expect(formatter('jack')).to.be.equals('hello jack');
    });
    it('should build compiler with valid message-format compiler function', function () {
        var messageFormatCompile = main_1.buildCompiler('en').messageFormat;
        var formatter = messageFormatCompile('hello {HUMAN}');
        chai_1.expect(formatter({ HUMAN: 'jack' })).to.be.equals('hello jack');
        var formatter2 = messageFormatCompile('hello {PLACE, selectordinal, other{#th}} human');
        chai_1.expect(formatter2({ PLACE: 4 })).to.be.equals('hello 4th human');
    });
    it('should build compiler with valid describe function', function () {
        var describe = main_1.buildCompiler('en').describe;
        chai_1.expect(describe({}).locale).to.be.equals('en');
        chai_1.expect(describe({ a: 'blah' }).tree.a).to.be.equals('blah');
    });
});
describe('format', function () {
    it('should format the string using the main language', function () {
        var language = { A: compiler.constant('B') };
        var str = main_1.format(language, null, 'A')();
        chai_1.expect(str).to.be.equals('B');
    });
    it('should format the string using the main language with data', function () {
        var language = { A: compiler.callback(function (d) { return 'B' + d.C; }) };
        var str = main_1.format(language, null, 'A')({ C: 'C' });
        chai_1.expect(str).to.be.equals('BC');
    });
    it('should format the string using the fallback language', function () {
        var language = { A: compiler.constant('B') };
        var str = main_1.format({}, language, 'A')();
        chai_1.expect(str).to.be.equals('B');
    });
    it('should format the string using the fallback language with data', function () {
        var language = { A: compiler.callback(function (d) { return 'B' + d.C; }) };
        var str = main_1.format({}, language, 'A')({ C: 'C' });
        chai_1.expect(str).to.be.equals('BC');
    });
    it('should format the string using the main language with bundle with data', function () {
        var language = { A: compiler.callback(function (d) { return 'B' + d.C; }) };
        var str = main_1.format({}, language, { data: { C: 'C' }, key: 'A' });
        chai_1.expect(str).to.be.equals('BC');
    });
    it('should fail if it finds no match', function () {
        chai_1.expect(function () { return main_1.format({}, {}, 'A')({ C: 'C' }); }).to.throw();
        chai_1.expect(function () { return main_1.format({}, null, 'A')({ C: 'C' }); }).to.throw();
    });
    it('should fail if it finds no match with bundle', function () {
        chai_1.expect(function () { return main_1.format({}, {}, { data: { C: 'C' }, key: 'A' }); }).to.throw();
        chai_1.expect(function () { return main_1.format({}, null, { data: { C: 'C' }, key: 'A' }); }).to.throw();
    });
    it('should fail if key or bundle is not provided', function () {
        chai_1.expect(function () { return main_1.format({}, {}, undefined); }).to.throw();
    });
    it('should not fail if bundle is null', function () {
        chai_1.expect(function () { return main_1.format({}, {}, null); }).not.to.throw();
    });
});
describe('buildFormatter', function () {
    it('should build valid formatter', function () {
        var language = { A: compiler.constant('B') };
        var language2 = { B: compiler.callback(function (d) { return 'B' + d.C; }) };
        var str = main_1.buildFormatter(language, language2);
        chai_1.expect(str('A')()).to.be.equals('B');
        chai_1.expect(str('B')({ C: 'C' })).to.be.equals('BC');
    });
});
var englishCompiler = main_1.buildCompiler('en');
var english = {
    locale: 'en',
    tree: {
        a: {
            b: {
                c: {
                    de: englishCompiler.constant('hello'),
                },
            },
        },
        c: englishCompiler.callback(function () { return 'bye'; }),
    },
};
var frenchCompiler = main_1.buildCompiler('fr');
var french = {
    locale: 'fr',
    tree: {
        a: {
            b: {
                c: {
                    de: frenchCompiler.constant('bonjour'),
                },
            },
        },
    },
};
describe('internationalize', function () {
    it('should properly process keys', function () {
        var i18n = main_1.internationalize(english, french);
        chai_1.expect(i18n.languages['en']['A_B_C_DE'].key).to.be.equals('A_B_C_DE');
        chai_1.expect(i18n.languages['en']['C'].key).to.be.equals('C');
    });
    it('should properly process path', function () {
        var i18n = main_1.internationalize(english, french);
        chai_1.expect(i18n.languages['en']['A_B_C_DE'].path).to.be.equals('a.b.c.de');
        chai_1.expect(i18n.languages['en']['C'].path).to.be.equals('c');
    });
    it('should properly assign the main locale to return object', function () {
        var i18n = main_1.internationalize(english, french);
        chai_1.expect(i18n.mainLocale).to.be.equals('en');
    });
    it('should properly expose the main language\'s tree', function () {
        var i18n = main_1.internationalize(english, french);
        chai_1.expect(i18n.mainTree.a.b.c.de.key).to.be.equals('A_B_C_DE');
    });
    it('should properly build the languages', function () {
        var i18n = main_1.internationalize(english, french);
        chai_1.expect(i18n.languages['en']['A_B_C_DE']).to.be.equals(english.tree.a.b.c.de);
        chai_1.expect(i18n.languages['en']['C']).to.be.equals(english.tree.c);
        chai_1.expect(i18n.languages['fr']['A_B_C_DE']).to.be.equals(french.tree.a.b.c.de);
    });
    it('should properly build format builder that works on the main language', function () {
        var i18n = main_1.internationalize(english, french);
        chai_1.expect(i18n.buildFormatter('en')('A_B_C_DE')()).to.be.equals('hello');
    });
    it('should properly build format builder that works on the other provided languages', function () {
        var i18n = main_1.internationalize(english, french);
        chai_1.expect(i18n.buildFormatter('fr')('A_B_C_DE')()).to.be.equals('bonjour');
    });
    it('should properly build format builder that works on ' +
        ' the other provided languages with main as fallback', function () {
        var i18n = main_1.internationalize(english, french);
        chai_1.expect(i18n.buildFormatter('fr')('C')()).to.be.equals('bye');
    });
});
