"use strict";
var chai_1 = require('chai');
var buildDescriptorsFromTree_1 = require('../src/buildDescriptorsFromTree');
var component = function () { return null; };
var reducerBuilder = function () { return function () { return 'state!'; }; };
var descriptors = buildDescriptorsFromTree_1.buildDescriptorsFromTree({
    'a': {
        contents: {
            'b': {
                contents: {
                    'c': {
                        component: component,
                        reducerBuilder: reducerBuilder,
                        titleKey: 'C',
                    },
                },
                titleKey: 'B',
            },
            'd': {
                contents: {
                    'e': {
                        component: component,
                        reducerBuilder: reducerBuilder,
                        titleKey: 'E',
                    },
                },
                titleKey: 'D',
            },
        },
        hidden: true,
        titleKey: 'A',
    },
    'f': {
        component: component,
        reducerBuilder: reducerBuilder,
        titleKey: 'F',
    },
    'g': {
        contents: {
            'h': {
                component: component,
                reducerBuilder: reducerBuilder,
                hidden: true,
                titleKey: 'H',
            },
        },
        titleKey: 'G',
    },
});
describe('buildDescriptorsFromTree', function () {
    it('should build valid page descriptors with correct path', function () {
        chai_1.expect(descriptors.pages.map(function (p) { return p.path; })).to.deep.equal([
            'a/b/c',
            'a/d/e',
            'f',
            'g/h',
        ]);
    });
    it('should build valid folder descriptors with correct path', function () {
        chai_1.expect(descriptors.folders.map(function (p) { return p.path; })).to.deep.equal([
            'a',
            'a/b',
            'a/d',
            'g',
        ]);
    });
    it('should properly propagate hidden', function () {
        chai_1.expect(Array.prototype.concat(descriptors.folders.filter(function (f) { return !!f.hidden; }).map(function (f) { return f.path; }), descriptors.pages.filter(function (p) { return p.hidden; }).map(function (p) { return p.path; }))).to.deep.equal([
            'a',
            'a/b',
            'a/d',
            'g',
            'a/b/c',
            'a/d/e',
            'g/h',
        ]);
    });
    it('should build valid path tree', function () {
        chai_1.expect(descriptors.pathTree).to.deep.equal([
            '',
            [
                'a',
                [
                    'a/b',
                    'a/b/c',
                ],
                [
                    'a/d',
                    'a/d/e',
                ],
            ],
            'f',
            [
                'g',
                'g/h',
            ],
        ]);
    });
    it('should build valid folder contents', function () {
        chai_1.expect(descriptors.folderContents).to.deep.equal({
            '': [
                'a/b/c',
                'a/d/e',
                'f',
                'g/h',
            ],
            'a': [
                'a/b/c',
                'a/d/e',
            ],
            'a/b': [
                'a/b/c',
            ],
            'a/d': [
                'a/d/e',
            ],
            'g': [
                'g/h',
            ],
        });
    });
    it('should build valid filtered path tree', function () {
        chai_1.expect(descriptors.filteredPathTree).to.deep.equal([
            '',
            'f',
        ]);
    });
    it('should build valid folder contents', function () {
        chai_1.expect(descriptors.filteredFolderContents).to.deep.equal({
            '': [
                'f',
            ],
        });
    });
    it('should augment the initial page and folder objects to include the path', function () {
        var tree = {
            'a': {
                contents: {
                    'b': {
                        contents: {
                            'c': {
                                titleKey: 'C',
                                component: component,
                                reducerBuilder: reducerBuilder,
                            },
                        },
                        titleKey: 'B',
                    },
                    'd': {
                        contents: {
                            'e': {
                                titleKey: 'E',
                                component: component,
                                reducerBuilder: reducerBuilder,
                            },
                        },
                        titleKey: 'D',
                    },
                },
                hidden: true,
                titleKey: 'A',
            },
        };
        buildDescriptorsFromTree_1.buildDescriptorsFromTree(tree);
        chai_1.expect(tree.a['path']).to.be.equals('a');
        chai_1.expect(tree.a.contents.b['path']).to.be.equals('a/b');
        chai_1.expect(tree.a.contents.b.contents.c['path']).to.be.equals('a/b/c');
        chai_1.expect(tree.a.contents.d['path']).to.be.equals('a/d');
        chai_1.expect(tree.a.contents.d.contents.e['path']).to.be.equals('a/d/e');
    });
});
