"use strict";
var chai_1 = require('chai');
var pages_1 = require('../src/pages');
var navigator_1 = require('../src/navigator');
var actions = require('../src/actions');
var persist = require('../src/persist');
var component = function () { return null; };
var reducerBuilder = function () { return function (s, _a) {
    if (s === void 0) { s = { value: 1 }; }
    var value = _a.value;
    return value ? { value: value } : s;
}; };
var tree = {
    'a': {
        contents: {
            'd': {
                contents: {
                    'e': {
                        component: component,
                        reducerBuilder: reducerBuilder,
                        hidden: true,
                        titleKey: 'E',
                    },
                },
                titleKey: 'D',
            },
            'g': {
                contents: {
                    'e': {
                        component: component,
                        reducerBuilder: reducerBuilder,
                        persistent: false,
                        titleKey: 'E',
                    },
                },
                titleKey: 'G',
            },
        },
        titleKey: 'A',
    },
    'f': {
        titleKey: 'F',
        component: component,
        reducerBuilder: reducerBuilder,
    },
};
var navigator = new navigator_1.Navigator(new pages_1.Pages(tree), 'ADMIN');
function check(nav, open, active, store) {
    if (store === void 0) { store = true; }
    chai_1.expect(nav.all.map(function (i) { return i.path; }).toArray()).to.deep.equal(open);
    if (active === null) {
        chai_1.expect(nav.active).to.be.null;
    }
    else {
        chai_1.expect(nav.active.mutex).to.be.equals(active);
    }
    chai_1.expect(nav.activeMutex).to.be.equals(active);
    if (store) {
        checkStore(nav, open, active);
    }
}
function checkStore(_, open, active) {
    chai_1.expect(persist.load('ADMIN').map(function (i) { return i.path; })).to.be.deep.equal(open);
    chai_1.expect(persist.loadActivePage('ADMIN')).to.be.equals(active);
}
describe('Navigator', function () {
    describe('dispatch/reduce', function () {
        describe('open', function () {
            it('should open a page and set active page to it', function () {
                persist.clear('ADMIN');
                var nav = navigator;
                var dispatch = function (a) { return nav = nav.reduce(a); };
                nav.open(dispatch, { mutex: '1', path: 'a/d/e' });
                check(nav, ['a/d/e'], '1');
            });
            it('should ignore pages with unknown path', function () {
                persist.clear('ADMIN');
                var nav = navigator;
                var dispatch = function (a) { return nav = nav.reduce(a); };
                nav.open(dispatch, { mutex: '1', path: 'blah1' });
                nav.open(dispatch, { mutex: '2', path: 'blah2' });
                nav.open(dispatch, { mutex: '3', path: 'blah3' });
                nav.open(dispatch, { mutex: '4', path: 'blah4' });
                check(nav, [], null);
                chai_1.expect(nav).to.be.equals(navigator);
            });
            it('should leave store unchanged when opening non persistent pages', function () {
                persist.clear('ADMIN');
                var nav = navigator;
                var dispatch = function (a) { return nav = nav.reduce(a); };
                nav.open(dispatch, { mutex: '1', path: 'a/d/e' });
                nav.open(dispatch, { mutex: '2', path: 'f' });
                nav.open(dispatch, { mutex: '3', path: 'a/d/e' });
                check(nav, ['a/d/e', 'f', 'a/d/e'], '3');
                nav.open(dispatch, { mutex: '4', path: 'a/g/e' });
                check(nav, ['a/d/e', 'f', 'a/d/e', 'a/g/e'], '4', false);
                checkStore(nav, ['a/d/e', 'f', 'a/d/e'], '3');
            });
            it('should transit to pages with already open mutex instead', function () {
                persist.clear('ADMIN');
                var nav = navigator;
                var dispatch = function (a) { return nav = nav.reduce(a); };
                nav.open(dispatch, { mutex: '1', path: 'a/d/e' });
                nav.open(dispatch, { mutex: '2', path: 'f' });
                nav.open(dispatch, { mutex: '3', path: 'a/d/e' });
                check(nav, ['a/d/e', 'f', 'a/d/e'], '3');
                nav.open(dispatch, { mutex: '1', path: 'a/d/e' });
                check(nav, ['a/d/e', 'f', 'a/d/e'], '1');
            });
        });
        describe('close', function () {
            var nav;
            var dispatch = function (a) { return nav = nav.reduce(a); };
            beforeEach('open pages for testing close', function () {
                persist.clear('ADMIN');
                nav = navigator;
                nav.open(dispatch, { mutex: '1', path: 'a/d/e' });
                nav.open(dispatch, { mutex: '2', path: 'f' });
                nav.open(dispatch, { mutex: '3', path: 'a/d/e' });
                nav.open(dispatch, { mutex: '4', path: 'f' });
                check(nav, ['a/d/e', 'f', 'a/d/e', 'f'], '4');
            });
            it('should ignore if the mutex is not open', function () {
                var previousNav = nav;
                nav.close(dispatch, 'blah');
                chai_1.expect(previousNav).to.be.equals(nav);
            });
            it('should close the page if it exists', function () {
                nav.close(dispatch, '2');
                check(nav, ['a/d/e', 'a/d/e', 'f'], '4');
                nav.close(dispatch, '3');
                check(nav, ['a/d/e', 'f'], '4');
                nav.close(dispatch, '1');
                nav.close(dispatch, '1');
                check(nav, ['f'], '4');
            });
            it('should leave store unchanged when closing non persistent pages', function () {
                nav.open(dispatch, { mutex: '5', path: 'a/g/e' });
                nav.open(dispatch, { mutex: '6', path: 'a/g/e' });
                check(nav, ['a/d/e', 'f', 'a/d/e', 'f', 'a/g/e', 'a/g/e'], '6', false);
                checkStore(nav, ['a/d/e', 'f', 'a/d/e', 'f'], '4');
                nav.transit(dispatch, '1');
                check(nav, ['a/d/e', 'f', 'a/d/e', 'f', 'a/g/e', 'a/g/e'], '1', false);
                checkStore(nav, ['a/d/e', 'f', 'a/d/e', 'f'], '1');
                nav.close(dispatch, '5');
                check(nav, ['a/d/e', 'f', 'a/d/e', 'f', 'a/g/e'], '1', false);
                checkStore(nav, ['a/d/e', 'f', 'a/d/e', 'f'], '1');
                nav.transit(dispatch, '6');
                check(nav, ['a/d/e', 'f', 'a/d/e', 'f', 'a/g/e'], '6', false);
                checkStore(nav, ['a/d/e', 'f', 'a/d/e', 'f'], '1');
                nav.close(dispatch, '6');
                check(nav, ['a/d/e', 'f', 'a/d/e', 'f'], '4', false);
                checkStore(nav, ['a/d/e', 'f', 'a/d/e', 'f'], '4');
            });
            it('should clear active page when active page targets a non persistent page', function () {
                nav.open(dispatch, { mutex: '5', path: 'a/g/e' });
                nav.open(dispatch, { mutex: '6', path: 'a/d/e' });
                check(nav, ['a/d/e', 'f', 'a/d/e', 'f', 'a/g/e', 'a/d/e'], '6', false);
                checkStore(nav, ['a/d/e', 'f', 'a/d/e', 'f', 'a/d/e'], '6');
                nav.close(dispatch, '6');
                check(nav, ['a/d/e', 'f', 'a/d/e', 'f', 'a/g/e'], '5', false);
                checkStore(nav, ['a/d/e', 'f', 'a/d/e', 'f'], null);
            });
            it('should properly change the active page after it has been closed', function () {
                nav.close(dispatch, '4');
                check(nav, ['a/d/e', 'f', 'a/d/e'], '3');
                nav.transit(dispatch, '1');
                check(nav, ['a/d/e', 'f', 'a/d/e'], '1');
                nav.close(dispatch, '1');
                check(nav, ['f', 'a/d/e'], '2');
                nav.close(dispatch, '2');
                check(nav, ['a/d/e'], '3');
            });
            it('should set active page to null if all pages are closed', function () {
                nav.close(dispatch, '1');
                nav.close(dispatch, '2');
                nav.close(dispatch, '3');
                nav.close(dispatch, '4');
                check(nav, [], null);
            });
        });
        describe('transit', function () {
            var nav;
            var dispatch = function (a) { return nav = nav.reduce(a); };
            beforeEach('open pages for testing close', function () {
                persist.clear('ADMIN');
                nav = navigator;
                nav.open(dispatch, { mutex: '1', path: 'a/d/e' });
                nav.open(dispatch, { mutex: '2', path: 'f' });
                nav.open(dispatch, { mutex: '3', path: 'a/d/e' });
                check(nav, ['a/d/e', 'f', 'a/d/e'], '3');
            });
            it('should ignore if the mutex is not open', function () {
                var previousNav = nav;
                nav.transit(dispatch, 'blah');
                chai_1.expect(previousNav).to.be.equals(nav);
            });
            it('should transit to null properly', function () {
                nav.transit(dispatch, null);
                check(nav, ['a/d/e', 'f', 'a/d/e'], null);
            });
            it('should leave store unchanged when transiting to non persistent pages', function () {
                nav.open(dispatch, { mutex: '4', path: 'a/g/e' });
                nav.open(dispatch, { mutex: '5', path: 'a/g/e' });
                check(nav, ['a/d/e', 'f', 'a/d/e', 'a/g/e', 'a/g/e'], '5', false);
                checkStore(nav, ['a/d/e', 'f', 'a/d/e'], '3');
                nav.transit(dispatch, '4');
                check(nav, ['a/d/e', 'f', 'a/d/e', 'a/g/e', 'a/g/e'], '4', false);
                checkStore(nav, ['a/d/e', 'f', 'a/d/e'], '3');
                nav.transit(dispatch, '5');
                check(nav, ['a/d/e', 'f', 'a/d/e', 'a/g/e', 'a/g/e'], '5', false);
                checkStore(nav, ['a/d/e', 'f', 'a/d/e'], '3');
                nav.transit(dispatch, '2');
                check(nav, ['a/d/e', 'f', 'a/d/e', 'a/g/e', 'a/g/e'], '2', false);
                checkStore(nav, ['a/d/e', 'f', 'a/d/e'], '2');
            });
            it('should peroperly transit to the target mutex', function () {
                nav.transit(dispatch, '1');
                check(nav, ['a/d/e', 'f', 'a/d/e'], '1');
                nav.transit(dispatch, '3');
                check(nav, ['a/d/e', 'f', 'a/d/e'], '3');
                nav.transit(dispatch, '2');
                check(nav, ['a/d/e', 'f', 'a/d/e'], '2');
            });
        });
        describe('clear', function () {
            it('should properly clear all and active pages', function () {
                var nav;
                var dispatch = function (a) { return nav = nav.reduce(a); };
                persist.clear('ADMIN');
                nav = navigator;
                nav.open(dispatch, { mutex: '1', path: 'a/d/e' });
                nav.open(dispatch, { mutex: '2', path: 'f' });
                nav.open(dispatch, { mutex: '3', path: 'a/d/e' });
                check(nav, ['a/d/e', 'f', 'a/d/e'], '3');
                nav.clear(dispatch);
                check(nav, [], null);
            });
        });
        describe('load', function () {
            it('should return self if persistent store is empty or has invalid paths', function () {
                var nav = navigator;
                var dispatch = function (a) { return nav = nav.reduce(a); };
                persist.clear('ADMIN');
                nav.load(dispatch);
                chai_1.expect(nav).to.be.equals(navigator);
                persist.save('ADMIN', [
                    { mutex: '1', path: 'blah!' },
                    { mutex: '2', path: 'blah!blah!' },
                    { mutex: '3', path: 'blah!blah!blah!' },
                ]);
                persist.saveActivePage('ADMIN', '2');
                nav.load(dispatch);
                chai_1.expect(nav).to.be.equals(navigator);
            });
            it('should properly load instances and the active page from persistent store', function () {
                var nav = navigator;
                var dispatch = function (a) { return nav = nav.reduce(a); };
                persist.clear('ADMIN');
                persist.save('ADMIN', [
                    { mutex: '1', path: 'a/d/e' },
                    { mutex: '2', path: 'f' },
                    { mutex: '3', path: 'a/d/e' },
                ]);
                persist.saveActivePage('ADMIN', '3');
                nav.load(dispatch);
                check(nav, ['a/d/e', 'f', 'a/d/e'], '3');
            });
            it('should ignore unknown paths', function () {
                var nav = navigator;
                var dispatch = function (a) { return nav = nav.reduce(a); };
                persist.clear('ADMIN');
                persist.save('ADMIN', [
                    { mutex: '1', path: 'a/d/e' },
                    { mutex: '2', path: 'blah!' },
                    { mutex: '3', path: 'a/d/e' },
                ]);
                persist.saveActivePage('ADMIN', '2');
                nav.load(dispatch);
                check(nav, ['a/d/e', 'a/d/e'], null, false);
            });
            it('should tolerate null active page', function () {
                var nav = navigator;
                var dispatch = function (a) { return nav = nav.reduce(a); };
                persist.clear('ADMIN');
                persist.save('ADMIN', [
                    { mutex: '1', path: 'a/d/e' },
                    { mutex: '2', path: 'a/d/e' },
                ]);
                nav.load(dispatch);
                check(nav, ['a/d/e', 'a/d/e'], null);
            });
        });
    });
    describe('composite reduction', function () {
        var nav;
        var dispatch = function (a) { return nav = nav.reduce(a); };
        beforeEach('open pages for testing close', function () {
            persist.clear('ADMIN');
            nav = navigator;
            nav.open(dispatch, { mutex: '1', path: 'a/d/e' });
            nav.open(dispatch, { mutex: '2', path: 'f' });
            nav.open(dispatch, { mutex: '3', path: 'a/d/e' });
            check(nav, ['a/d/e', 'f', 'a/d/e'], '3');
        });
        it('should properly initialize page states', function () {
            nav.all.forEach(function (page) { return chai_1.expect(page.state).to.be.deep.equal({ value: 1 }); });
        });
        it('should call reduce only on active page', function () {
            var newNav = nav.reduce({ type: 'STUFF!', value: 10 });
            chai_1.expect(nav.active.state.value).to.be.equals(1);
            chai_1.expect(newNav.active.state.value).to.be.equals(10);
            chai_1.expect(newNav.all.get('1').state.value).to.be.equals(1);
            chai_1.expect(newNav.all.get('2').state.value).to.be.equals(1);
        });
        it('should call reduce on all pages if the action is deisolated', function () {
            var newNav = nav.reduce(navigator_1.deisolate({ type: 'STUFF!', value: 10 }));
            chai_1.expect(nav.active.state.value).to.be.equals(1);
            chai_1.expect(newNav.active.state.value).to.be.equals(10);
            chai_1.expect(newNav.all.get('1').state.value).to.be.equals(10);
            chai_1.expect(newNav.all.get('2').state.value).to.be.equals(10);
            chai_1.expect(newNav.all.get('3').state.value).to.be.equals(10);
        });
        it('should absorb actions that target it', function () {
            var action1 = actions.NAVIGATION_CLOSE_PAGE.create.unicast('ADMIN', '2');
            action1['value'] = 10;
            var newNav1 = nav.reduce(action1);
            chai_1.expect(nav.active.state.value).to.be.equals(1);
            chai_1.expect(newNav1.active.state.value).to.be.equals(1);
            var action2 = actions.NAVIGATION_OPEN_PAGE.create.unicast('ADMIN', {
                mutex: '3',
                path: 'a/d/e',
            });
            action2['value'] = 10;
            var newNav2 = nav.reduce(action2);
            chai_1.expect(nav.active.state.value).to.be.equals(1);
            chai_1.expect(newNav2.active.state.value).to.be.equals(1);
            var action3 = actions.NAVIGATION_TRANSIT_TO_PAGE.create.unicast('ADMIN', '3');
            action3['value'] = 10;
            var newNav3 = nav.reduce(action3);
            chai_1.expect(nav.active.state.value).to.be.equals(1);
            chai_1.expect(newNav3.active.state.value).to.be.equals(1);
        });
    });
    describe('all', function () {
        it('should return an empty map', function () {
            chai_1.expect(navigator.all.map(function (ins) { return ins.path; }).toArray()).to.deep.equal([]);
        });
        it('should return all open pages in correct order', function () {
            persist.clear('ADMIN');
            var nav = navigator;
            var dispatch = function (a) { return nav = nav.reduce(a); };
            nav.open(dispatch, { mutex: '1', path: 'a/d/e' });
            nav.open(dispatch, { mutex: '2', path: 'a/d/e' });
            nav.open(dispatch, { mutex: '3', path: 'f' });
            nav.open(dispatch, { mutex: '4', path: 'a/d/e' });
            nav.open(dispatch, { mutex: '2', path: 'a/d/e' });
            nav.open(dispatch, { mutex: '3', path: 'f' });
            nav.open(dispatch, { mutex: '5', path: 'f' });
            chai_1.expect(nav.all.map(function (ins) { return ins.path; }).toArray()).to.deep.equal([
                'a/d/e',
                'a/d/e',
                'f',
                'a/d/e',
                'f',
            ]);
        });
    });
    describe('active', function () {
        it('should return null when there are no active pages', function () {
            chai_1.expect(navigator.active).to.be.null;
        });
        it('should return the current active page', function () {
            persist.clear('ADMIN');
            var nav = navigator;
            var dispatch = function (a) { return nav = nav.reduce(a); };
            nav.open(dispatch, { mutex: '5', path: 'f' });
            chai_1.expect(nav.active.mutex).to.be.equals('5');
        });
    });
    describe('activeMutex', function () {
        it('should return null when there are no active pages', function () {
            chai_1.expect(navigator.activeMutex).to.be.null;
        });
        it('should return the current active page\' mutex', function () {
            persist.clear('ADMIN');
            var nav = navigator;
            var dispatch = function (a) { return nav = nav.reduce(a); };
            nav.open(dispatch, { mutex: '5', path: 'f' });
            chai_1.expect(nav.activeMutex).to.be.equals('5');
        });
    });
});
