"use strict";
var TRANSFORMER_ERROR = 'The provided transformer is not a function';
function composeTransformers(transformers) {
    if (!transformers) {
        return function (action) { return action; };
    }
    if (Array.isArray(transformers)) {
        transformers.forEach(function (t) {
            if (typeof t !== 'function') {
                throw new TypeError(TRANSFORMER_ERROR);
            }
        });
        if (transformers.length === 0) {
            return function (action) { return action; };
        }
        if (transformers.length === 1) {
            return transformers[0];
        }
        return function (action) { return transformers.reduce(function (prev, curr) { return curr(prev); }, action); };
    }
    if (typeof transformers === 'function') {
        return transformers;
    }
    throw new TypeError(TRANSFORMER_ERROR);
}
exports.composeTransformers = composeTransformers;
function createActionDescriptor(type, transformers) {
    var transform = composeTransformers(transformers);
    return {
        type: type,
        create: function (payload, meta) {
            return transform({
                type: type,
                payload: payload,
                meta: meta,
            });
        },
        is: function (action) {
            return action.type === type;
        },
    };
}
exports.createActionDescriptor = createActionDescriptor;
function createTargetedActionDescriptor(type, transformers) {
    var transform = composeTransformers(transformers);
    return {
        type: type,
        create: {
            unicast: function (target, payload, meta) {
                return transform({
                    type: type,
                    payload: payload,
                    meta: meta,
                    target: target,
                    broadcast: false,
                });
            },
            multicast: function (targets, payload, meta) {
                return transform({
                    type: type,
                    payload: payload,
                    meta: meta,
                    broadcast: false,
                    target: targets,
                });
            },
            broadcast: function (payload, meta) {
                return transform({
                    type: type,
                    payload: payload,
                    meta: meta,
                    broadcast: true,
                });
            },
        },
        is: function (id, action) {
            if (action.type !== type) {
                return false;
            }
            var targeted = action;
            if (targeted.broadcast) {
                return true;
            }
            if (Array.isArray(targeted.target)) {
                return targeted.target.indexOf(id) !== -1;
            }
            return targeted.target === id;
        },
    };
}
exports.createTargetedActionDescriptor = createTargetedActionDescriptor;
function createAsyncAction(resolve, success, fail, attempt) {
    return {
        create: function (payload, meta) { return function (dispatch) {
            if (attempt) {
                dispatch(attempt.create(payload, meta));
            }
            var promise = resolve(payload).then(function (result) { return dispatch(success.create(result, meta)); });
            if (fail) {
                promise.catch(function (error) { return dispatch(fail.create(error, meta)); });
            }
            return promise;
        }; },
    };
}
exports.createAsyncAction = createAsyncAction;
function hook(dispatch) {
    var transformers = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        transformers[_i - 1] = arguments[_i];
    }
    var transform = composeTransformers(transformers);
    return (function (action) {
        if (typeof action === 'function') {
            throw new TypeError('cannot hook into thunks');
        }
        if (typeof action !== 'object' || action === null) {
            throw new TypeError('invalid action provided to hook into');
        }
        dispatch(transform(action));
    });
}
exports.hook = hook;
