"use strict";
var chai_1 = require('chai');
var Bluebird = require('bluebird');
var action_1 = require('./action');
describe('composeTransformers', function () {
    it('should throw if called with non falsy non function values', function () {
        chai_1.expect(function () { return action_1.composeTransformers({}); }).to.throw();
    });
    it('should return identity function if no arguments or and empty array is provided', function () {
        var act = {};
        var identity = action_1.composeTransformers();
        chai_1.expect(act).to.be.equals(identity(act));
        var identity2 = action_1.composeTransformers([]);
        chai_1.expect(act).to.be.equals(identity2(act));
    });
    it('should return the single transformer only if one is provided', function () {
        var typeOnly = function (a) { return ({ type: a.type }); };
        var singular = action_1.composeTransformers(typeOnly);
        var singularByArray = action_1.composeTransformers([typeOnly]);
        chai_1.expect(typeOnly).to.be.equals(singular);
        chai_1.expect(typeOnly).to.be.equals(singularByArray);
    });
    it('should fail when a transformer is not valid', function () {
        chai_1.expect(function () { return action_1.composeTransformers([1]); }).to.throw();
        chai_1.expect(function () { return action_1.composeTransformers([function (a) { return a; }, 1]); }).to.throw();
    });
    it('should properly compose transformers', function () {
        var t1 = function (a) { a['2'] = a['1'] + 1; return a; };
        var t2 = function (a) { a['3'] = a['2'] + 1; return a; };
        var t3 = function (a) { a['4'] = a['3'] + 1; return a; };
        var t4 = function (a) { a['5'] = a['4'] + 1; return a; };
        var composed = action_1.composeTransformers([t1, t2, t3, t4]);
        chai_1.expect(composed({ type: 'A', 1: 1 })['5']).to.be.equals(5);
    });
});
describe('hook', function () {
    it('should properly dispatch a transformed action', function () {
        var t1 = function (a) { a['2'] = a['1'] + 1; return a; };
        var t2 = function (a) { a['3'] = a['2'] + 1; return a; };
        var t3 = function (a) { a['4'] = a['3'] + 1; return a; };
        var t4 = function (a) { a['5'] = a['4'] + 1; return a; };
        var action = null;
        var enhancedDispatch = action_1.hook(function (a) { return action = a; }, t1, t2, t3, t4);
        enhancedDispatch({ type: 'A', 1: 1 });
        chai_1.expect(action['5']).to.be.equals(5);
    });
    it('should fail if provided a thunk', function () {
        var enhancedDispatch = action_1.hook(function () { return null; });
        chai_1.expect(function () { return enhancedDispatch(function () { return null; }); }).to.throw();
    });
    it('should fail if provided an invalid action', function () {
        var enhancedDispatch = action_1.hook(function () { return null; });
        chai_1.expect(function () { return enhancedDispatch(null); }).to.throw();
    });
});
describe('createActionDescriptor', function () {
    it('should create an actionDescriptor with correct type', function () {
        var ACTION = action_1.createActionDescriptor('ACTION');
        chai_1.expect(ACTION.type).to.be.equals('ACTION');
    });
    it('should create an actionDescriptor with correct type and transformer', function () {
        var ACTION = action_1.createActionDescriptor('ACTION', [
            function (a) { return ({ type: a.type + '_1' }); },
            function (a) { return ({ type: a.type + '_2' }); },
            function (a) { return ({ type: a.type + '_3' }); },
        ]);
        chai_1.expect(ACTION.type).to.be.equals('ACTION');
        chai_1.expect(ACTION.create().type).to.be.equals('ACTION_1_2_3');
    });
    it('should create an actionDescriptor with functional create function', function () {
        var ACTION = action_1.createActionDescriptor('ACTION');
        var act = ACTION.create({ a: 'hello' }, { b: 'world' });
        chai_1.expect(act.payload.a).to.be.equals('hello');
        chai_1.expect(act.meta.b).to.be.equals('world');
        chai_1.expect(act.type).to.be.equals('ACTION');
    });
    it('should create an actionDescriptor with functional is function', function () {
        var ACTION = action_1.createActionDescriptor('ACTION');
        var act = ACTION.create({ a: 'hello' }, { b: 'world' });
        chai_1.expect(ACTION.is(act)).to.be.true;
    });
});
describe('createTargetedActionDescriptor', function () {
    it('should create a targetedActionDescriptor with correct type', function () {
        var ACTION = action_1.createTargetedActionDescriptor('ACTION');
        chai_1.expect(ACTION.type).to.be.equals('ACTION');
    });
    it('should create a targetedActionDescriptor with correct type and transformer', function () {
        var ACTION = action_1.createTargetedActionDescriptor('ACTION', [
            function (a) { return ({ type: a.type + '_1' }); },
            function (a) { return ({ type: a.type + '_2' }); },
            function (a) { return ({ type: a.type + '_3' }); },
        ]);
        chai_1.expect(ACTION.type).to.be.equals('ACTION');
        chai_1.expect(ACTION.create.unicast(1).type).to.be.equals('ACTION_1_2_3');
        chai_1.expect(ACTION.create.multicast([1]).type).to.be.equals('ACTION_1_2_3');
        chai_1.expect(ACTION.create.broadcast().type).to.be.equals('ACTION_1_2_3');
    });
    it('should create a targetedActionDescriptor with functional create unicast function', function () {
        var ACTION = action_1.createTargetedActionDescriptor('ACTION');
        var act = ACTION.create.unicast(1, { a: 'hello' }, { b: 'world' });
        chai_1.expect(act.payload.a).to.be.equals('hello');
        chai_1.expect(act.meta.b).to.be.equals('world');
        chai_1.expect(act.broadcast).to.be.false;
        chai_1.expect(act.target).to.be.equals(1);
    });
    it('should create a targetedActionDescriptor with functional create multicast function', function () {
        var ACTION = action_1.createTargetedActionDescriptor('ACTION');
        var act = ACTION.create.multicast([1, '2'], { a: 'hello' }, { b: 'world' });
        chai_1.expect(act.payload.a).to.be.equals('hello');
        chai_1.expect(act.meta.b).to.be.equals('world');
        chai_1.expect(act.broadcast).to.be.false;
        chai_1.expect(act.target).to.be.deep.equal([1, '2']);
    });
    it('should create a targetedActionDescriptor with functional create broadcast function', function () {
        var ACTION = action_1.createTargetedActionDescriptor('ACTION');
        var act = ACTION.create.broadcast({ a: 'hello' }, { b: 'world' });
        chai_1.expect(act.payload.a).to.be.equals('hello');
        chai_1.expect(act.meta.b).to.be.equals('world');
        chai_1.expect(act.broadcast).to.be.true;
        chai_1.expect(act.target).to.be.undefined;
    });
    describe('is', function () {
        it('should pass for unicast when target is id', function () {
            var ACTION = action_1.createTargetedActionDescriptor('ACTION');
            var act = ACTION.create.unicast(2, { a: 'hello' }, { b: 'world' });
            chai_1.expect(ACTION.is(2, act)).to.be.true;
        });
        it('should fail for unicast when target is not id', function () {
            var ACTION = action_1.createTargetedActionDescriptor('ACTION');
            var act = ACTION.create.unicast(2, { a: 'hello' }, { b: 'world' });
            chai_1.expect(ACTION.is(1, act)).to.be.false;
        });
        it('should pass for multicast when id is in targets', function () {
            var ACTION = action_1.createTargetedActionDescriptor('ACTION');
            var act = ACTION.create.multicast([1, 2], { a: 'hello' }, { b: 'world' });
            chai_1.expect(ACTION.is(2, act)).to.be.true;
        });
        it('should fail for multicast when id is not in targets', function () {
            var ACTION = action_1.createTargetedActionDescriptor('ACTION');
            var act = ACTION.create.multicast([1, 2], { a: 'hello' }, { b: 'world' });
            chai_1.expect(ACTION.is(3, act)).to.be.false;
        });
        it('should pass for broadcast whatever the id is', function () {
            var ACTION = action_1.createTargetedActionDescriptor('ACTION');
            var act = ACTION.create.broadcast({ a: 'hello' }, { b: 'world' });
            chai_1.expect(ACTION.is(2, act)).to.be.true;
        });
        it('should fail for type mismatch', function () {
            var ACTION = action_1.createTargetedActionDescriptor('ACTION');
            var act1 = ACTION.create.unicast(2, { a: 'hello' }, { b: 'world' });
            act1.type = 'STUFF1';
            chai_1.expect(ACTION.is(2, act1)).to.be.false;
            var act2 = ACTION.create.multicast([1, 2], { a: 'hello' }, { b: 'world' });
            act2.type = 'STUFF2';
            chai_1.expect(ACTION.is(2, act2)).to.be.false;
            var act3 = ACTION.create.broadcast({ a: 'hello' }, { b: 'world' });
            act3.type = 'STUFF3';
            chai_1.expect(ACTION.is(2, act3)).to.be.false;
        });
    });
});
describe('createAsyncAction', function () {
    it('should create an async action with expected behavior', function () {
        var SUCCESS = action_1.createActionDescriptor('SUCCESS');
        var ACTIONASYNC = action_1.createAsyncAction(function () { return Bluebird.resolve(1); }, SUCCESS);
        var act = ACTIONASYNC.create();
        var successCalledWith = 0;
        var dispatch = (function (action) {
            if (SUCCESS.is(action)) {
                successCalledWith = action.payload;
            }
        });
        return act(dispatch).then(function () { return chai_1.expect(successCalledWith).to.be.equals(1); });
    });
    it('should create an async action which properly calls attempt', function () {
        var SUCCESS = action_1.createActionDescriptor('SUCCESS');
        var ATTEMPT = action_1.createActionDescriptor('ATTEMPT');
        var ACTIONASYNC = action_1.createAsyncAction(function () { return Bluebird.resolve(1); }, SUCCESS, null, ATTEMPT);
        var act = ACTIONASYNC.create();
        var attemptCalled = false;
        var dispatch = (function (action) {
            if (ATTEMPT.is(action)) {
                attemptCalled = true;
            }
        });
        return act(dispatch).then(function () { return chai_1.expect(attemptCalled).to.be.true; });
    });
    it('should create an async action which properly calls fail', function () {
        var SUCCESS = action_1.createActionDescriptor('SUCCESS');
        var FAIL = action_1.createActionDescriptor('FAIL');
        var ACTIONASYNC = action_1.createAsyncAction(function () { return Bluebird.reject(new Error()); }, SUCCESS, FAIL);
        var act = ACTIONASYNC.create();
        var hasError = false;
        var dispatch = (function (action) {
            if (FAIL.is(action)) {
                hasError = true;
            }
        });
        return act(dispatch).catch(function () { return chai_1.expect(hasError).to.be.true; });
    });
});
