"use strict";
var chai_1 = require('chai');
var reducer_1 = require('./reducer');
var ReduceMe = (function () {
    function ReduceMe(initialized, changing, payload) {
        if (initialized === void 0) { initialized = false; }
        if (changing === void 0) { changing = false; }
        if (payload === void 0) { payload = null; }
        this.changing = changing;
        this.initialized = initialized;
        this.payload = payload;
    }
    ReduceMe.prototype.reduce = function (action) {
        if (this.changing) {
            return new ReduceMe(this.initialized, true);
        }
        if (action.type === 'INIT') {
            return new ReduceMe(true, this.changing);
        }
        if (action.payload) {
            return new ReduceMe(this.initialized, this.changing, action.payload);
        }
        return this;
    };
    return ReduceMe;
}());
describe('createAutoReducer', function () {
    it('should create an autoReducer that properly initializes state', function () {
        var initialState = {
            a: new ReduceMe(false),
        };
        var reducer = reducer_1.createAutoReducer(initialState);
        var initial = reducer(undefined, { type: 'INIT' });
        chai_1.expect(initialState.a.initialized).to.be.false;
        chai_1.expect(initial.a.initialized).to.be.true;
    });
    it('should create an autoReducer that does not return a new object if input is unchanged', function () {
        var reducer = reducer_1.createAutoReducer({
            a: new ReduceMe(),
            b: new ReduceMe(),
            c: new ReduceMe(),
        });
        var initial = reducer(undefined, { type: 'INIT' });
        var second = reducer(initial, { type: 'OTHER' });
        var third = reducer(initial, { type: 'SOME_OTHER' });
        chai_1.expect(initial).to.be.equals(second);
        chai_1.expect(second).to.be.equals(third);
    });
    it('should create an autoReducer that should properly handle action payloads', function () {
        var reducer = reducer_1.createAutoReducer({
            a: new ReduceMe(),
        });
        var initial = reducer(undefined, { type: 'INIT' });
        var second = reducer(initial, { payload: true, type: 'OTHER' });
        chai_1.expect(initial.a.payload).to.be.null;
        chai_1.expect(second.a.payload).to.be.true;
    });
});
describe('createMutatorReducer', function () {
    it('should create a mutatorReducer that properly initializes state', function () {
        var initialState = { a: ' ', b: 1 };
        var reducer = reducer_1.createMutatorReducer(initialState, function () { return null; });
        var state = reducer(undefined, { type: 'INIT' });
        chai_1.expect(state).to.be.equals(initialState);
    });
    it('should create a mutatorReducer that returns the same object if unmodified', function () {
        var initialState = { a: ' ', b: 1 };
        var reducer = reducer_1.createMutatorReducer(initialState, function (_, __, set) {
            set('a', ' ');
            set('b', 1);
        });
        var first = reducer(undefined, { type: 'INIT' });
        var second = reducer(first, { type: 'OTHER' });
        chai_1.expect(first).to.be.equals(initialState);
        chai_1.expect(second).to.be.equals(initialState);
    });
    it('should create a mutatorReducer that correctly sets the props', function () {
        var initialState = { a: ' ', b: 1 };
        var reducer = reducer_1.createMutatorReducer(initialState, function (_, __, set) {
            set('a', 'foo');
            set('b', 2);
        });
        var first = reducer(undefined, { type: 'INIT' });
        var second = reducer(first, { type: 'OTHER' });
        chai_1.expect(first).not.to.be.equals(initialState);
        chai_1.expect(second).to.be.equals(first);
        chai_1.expect(second.a).to.be.equals('foo');
        chai_1.expect(second.b).to.be.equals(2);
    });
    it('should create a mutatorReducer that correctly sets the props when not defined', function () {
        var initialState = { a: ' ', b: 1 };
        var reducer = reducer_1.createMutatorReducer(initialState, function (_, __, set) {
            set('c', true);
        });
        var state = reducer(undefined, { type: 'INIT' });
        chai_1.expect(state).not.to.be.equals(initialState);
        chai_1.expect(state.c).to.be.true;
    });
    it('should create a mutatorReducer that correctly unsets the props', function () {
        var initialState = { a: ' ', b: 1 };
        var reducer = reducer_1.createMutatorReducer(initialState, function (_, __, set, unset) {
            set('a', 1);
            unset('a');
        });
        var state = reducer(undefined, { type: 'INIT' });
        chai_1.expect(state).not.to.be.equals(initialState);
        chai_1.expect(state.b).to.be.equals(1);
        chai_1.expect(state.a).to.be.undefined;
    });
    it('should create a mutatorReducer that correctly resets a prop after it was unset', function () {
        var initialState = { a: ' ', b: 1 };
        var reducer = reducer_1.createMutatorReducer(initialState, function (_, __, set, unset) {
            unset('a');
            set('a', 'foo');
        });
        var state = reducer(undefined, { type: 'INIT' });
        chai_1.expect(state).not.to.be.equals(initialState);
        chai_1.expect(state.b).to.be.equals(1);
        chai_1.expect(state.a).to.be.equals('foo');
    });
    it('should create a mutatorReducer that correctly resets the state', function () {
        var initialState = { a: ' ', b: 1 };
        var reducer = reducer_1.createMutatorReducer(initialState, function (_, action, set, __, reset) {
            set('a', 'foo');
            if (action.type === 'RESET') {
                reset();
            }
        });
        var first = reducer(undefined, { type: 'INIT' });
        var second = reducer(first, { type: 'RESET' });
        chai_1.expect(first).not.to.be.equals(initialState);
        chai_1.expect(second).not.to.be.equals(first);
        chai_1.expect(second).to.be.equals(initialState);
    });
    it('should create a mutatorReducer that resets the state to the specified object', function () {
        var initialState = { a: ' ', b: 1 };
        var otherState = { a: 'bar', b: 2 };
        var reducer = reducer_1.createMutatorReducer(initialState, function (_, action, set, __, reset) {
            set('a', 'foo');
            if (action.type === 'RESET') {
                reset(otherState);
            }
        });
        var first = reducer(undefined, { type: 'INIT' });
        var second = reducer(first, { type: 'RESET' });
        chai_1.expect(second).not.to.be.equals(first);
        chai_1.expect(second).to.be.equals(otherState);
        chai_1.expect(second.a).to.be.equals('bar');
    });
});
describe('createReducibleReducer', function () {
    it('should create a reducibleReducer that properly initializes state', function () {
        var initialState = new ReduceMe(false);
        var reducer = reducer_1.createReducibleReducer(initialState);
        var initial = reducer(undefined, { type: 'INIT' });
        chai_1.expect(initialState.initialized).to.be.false;
        chai_1.expect(initial.initialized).to.be.true;
    });
    it('should create a reducibleReducer that only returns a new object if input is changed', function () {
        var reducer = reducer_1.createReducibleReducer(new ReduceMe());
        var initial = reducer(undefined, { type: 'INIT' });
        var second = reducer(initial, { type: 'OTHER' });
        var third = reducer(initial, { type: 'SOME_OTHER' });
        chai_1.expect(initial).to.be.equals(second);
        chai_1.expect(second).to.be.equals(third);
    });
    it('should create a reducibleReducer that should properly handle action payloads', function () {
        var reducer = reducer_1.createReducibleReducer(new ReduceMe());
        var initial = reducer(undefined, { type: 'INIT' });
        var second = reducer(initial, { payload: true, type: 'OTHER' });
        chai_1.expect(initial.payload).to.be.null;
        chai_1.expect(second.payload).to.be.true;
    });
});
describe('createConstantReducer', function () {
    it('should create a constantReducer that properly initializes state', function () {
        var reducer = reducer_1.createConstantReducer(1);
        var initial = reducer.reduce({ type: 'INIT' });
        chai_1.expect(initial.value).to.be.equals(1);
    });
    it('should create a constantReducer that can act as auto reducer', function () {
        var reducer = reducer_1.createAutoReducer({ constant: reducer_1.createConstantReducer(1) });
        var initial = reducer(undefined, { type: 'TYPE' });
        chai_1.expect(initial.constant.value).to.be.equals(1);
    });
});
