"use strict";
var chai_1 = require('chai');
var attribute_1 = require('./attribute');
function mockModel(name) {
    return {
        name: name,
        attributes: [],
        indexes: { unique: [] },
        join: false,
        owner: null,
        relations: [],
        virtual: false,
    };
}
describe('attribute', function () {
    describe('validateAttribute', function () {
        it('should fail when name is invalid', function () {
            var model = mockModel('A');
            var t = attribute_1.DataTypes.INTEGER();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, { name: null, type: t }); }).to.throw();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, { name: 1, type: t }); }).to.throw();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, { name: 0, type: t }); }).to.throw();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, { name: false, type: t }); }).to.throw();
        });
        it('should fail when type is invalid', function () {
            var model = mockModel('A');
            var t = attribute_1.DataTypes.INTEGER;
            chai_1.expect(function () { return attribute_1.appendAttribute(model, { name: 'a', type: t }); }).to.throw();
        });
        it('should fail when autoIncrement is used with non-primary', function () {
            var model = mockModel('A');
            var t = attribute_1.DataTypes.INTEGER();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, {
                autoIncrement: true,
                name: null,
                primary: false,
                type: t,
            }); }).to.throw();
        });
        it('should fail when autoIncrement is used with non-integers', function () {
            var model = mockModel('A');
            var t = attribute_1.DataTypes.STRING();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, {
                autoIncrement: true,
                name: null,
                primary: true,
                type: t,
            }); }).to.throw();
        });
        it('should fail when primary is used with either unique or require or both', function () {
            var model = mockModel('A');
            var t = attribute_1.DataTypes.INTEGER();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, {
                name: 'a',
                primary: true,
                required: false,
                type: t,
                unique: false,
            }); }).not.to.throw();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, {
                name: 'b',
                primary: true,
                type: t,
                unique: true,
            }); }).to.throw();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, {
                name: 'c',
                primary: true,
                required: true,
                type: t,
            }); }).to.throw();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, {
                name: 'd',
                primary: true,
                required: true,
                type: t,
                unique: true,
            }); }).to.throw();
        });
        it('should fail when validator is provided and is not a function', function () {
            var model = mockModel('A');
            var t = attribute_1.DataTypes.INTEGER();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, {
                name: 'a',
                type: t,
                validator: 'blah',
            }); }).to.throw();
        });
        it('should fail when the validator cannot validate the provided default value', function () {
            var model = mockModel('A');
            var t = attribute_1.DataTypes.INTEGER();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, {
                defaultValue: 1,
                name: 'a',
                type: t,
                validator: function (n) { return n > 0 ? { data: null, key: 'A' } : null; },
            }); }).to.throw();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, {
                defaultValue: null,
                name: 'a',
                type: t,
                validator: function (n) { return n === null ? { data: null, key: 'A' } : null; },
            }); }).to.throw();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, {
                name: 'a',
                type: t,
                validator: function (n) { return n > 0 ? { data: null, key: 'A' } : null; },
            }); }).not.to.throw();
        });
    });
    describe('appendAttribute', function () {
        it('should properly append attributes to the model', function () {
            var model = mockModel('A');
            var type = attribute_1.DataTypes.INTEGER();
            attribute_1.appendAttribute(model, { name: 'a', type: type });
            attribute_1.appendAttribute(model, { name: 'a', type: null }, { name: 'b', type: type });
            attribute_1.appendAttribute(model, {}, { name: 'c', type: type });
            chai_1.expect(model.attributes[0].name).to.be.equals('a');
            chai_1.expect(model.attributes[1].name).to.be.equals('b');
            chai_1.expect(model.attributes[2].name).to.be.equals('c');
        });
        it('should fail if no model is provided', function () {
            chai_1.expect(function () { return attribute_1.appendAttribute(null, {}); }).to.throw();
        });
        it('should fail if no attribute is provided', function () {
            chai_1.expect(function () { return attribute_1.appendAttribute(mockModel('A')); }).to.throw();
        });
        it('should fail if 2 attributes with the same name are added', function () {
            var model = mockModel('A');
            var type = attribute_1.DataTypes.INTEGER();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, { name: 'a', type: type }); }).not.to.throw();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, { name: 'a', type: type }); }).to.throw();
        });
        it('should fail if 2 or more attributes are marked primary', function () {
            var model = mockModel('A');
            var type = attribute_1.DataTypes.INTEGER();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, { name: 'a', type: type, primary: true }); }).not.to.throw();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, { name: 'b', type: type, primary: true }); }).to.throw();
            chai_1.expect(function () { return attribute_1.appendAttribute(model, { name: 'c', type: type, primary: true }); }).to.throw();
        });
    });
    describe('createAttributeBuilder', function () {
        it('should fail if no model is provided', function () {
            chai_1.expect(function () { return attribute_1.createAttributeBuilder(null); }).to.throw();
        });
        describe('attributeBuilder', function () {
            it('should properly build a valid attribute', function () {
                var builder = attribute_1.createAttributeBuilder(mockModel('a'));
                chai_1.expect(function () { return builder({
                    name: 'a',
                    type: attribute_1.DataTypes.INTEGER(),
                }); }).not.to.throw();
            });
            it('should properly build a valid integer primary key attribute', function () {
                var builder = attribute_1.createAttributeBuilder(mockModel('a'));
                chai_1.expect(function () { return builder.primaryKey({
                    name: 'a',
                    type: attribute_1.DataTypes.INTEGER(),
                }); }).not.to.throw();
            });
            it('should properly build a valid manual integer primary key attribute', function () {
                var model = mockModel('a');
                var builder = attribute_1.createAttributeBuilder(model);
                chai_1.expect(function () { return builder.primaryKey({
                    autoIncrement: false,
                    name: 'a',
                    type: attribute_1.DataTypes.INTEGER(),
                }); }).not.to.throw();
                chai_1.expect(model.attributes[0].autoIncrement).to.be.false;
            });
            it('should properly build a valid string primary key attribute', function () {
                var builder = attribute_1.createAttributeBuilder(mockModel('a'));
                chai_1.expect(function () { return builder.primaryKey({
                    name: 'b',
                    type: attribute_1.DataTypes.STRING(),
                }); }).not.to.throw();
            });
        });
    });
});
