"use strict";
var chai_1 = require('chai');
var attribute_1 = require('./attribute');
var joinModel_1 = require('./joinModel');
function mockModel(name) {
    return {
        name: name,
        attributes: [],
        indexes: { unique: [] },
        join: false,
        owner: null,
        relations: [],
        virtual: false,
    };
}
function mockUninitializedStateModel(model) {
    return {
        model: model,
        initialize: function () { return null; },
        join: model.join,
        name: model.name,
        owner: model.owner,
        virtual: model.virtual,
    };
}
describe('joinModel', function () {
    describe('joinee', function () {
        it('should fail if provided invalid input', function () {
            var model = mockUninitializedStateModel(mockModel('A'));
            chai_1.expect(function () { return joinModel_1.joinee(null, 'a', 'all_a'); }).to.throw();
            chai_1.expect(function () { return joinModel_1.joinee(model, null, 'all_a'); }).to.throw();
            chai_1.expect(function () { return joinModel_1.joinee(model, 'a', null); }).to.throw();
        });
        it('should return a joinee with valid defaults', function () {
            var model = mockUninitializedStateModel(mockModel('A'));
            var joineeModel = joinModel_1.joinee(model, 'a', 'all_a');
            chai_1.expect(joineeModel.aliasOnJoin).to.be.equals('a');
            chai_1.expect(joineeModel.aliasOnTarget).to.be.equals('all_a');
            chai_1.expect(joineeModel.key).to.be.equals('aId');
            chai_1.expect(joineeModel.model).to.be.equals(model);
            chai_1.expect(joineeModel.onDelete).to.be.equals('CASCADE');
            chai_1.expect(joineeModel.onUpdate).to.be.equals('CASCADE');
            chai_1.expect(joineeModel.type).to.be.instanceof(attribute_1.IntegerType);
        });
        it('should return a joinee with overriden defaults', function () {
            var model = mockUninitializedStateModel(mockModel('A'));
            var joineeModel = joinModel_1.joinee(model, 'a', 'all_a', {
                key: 'someId',
                onDelete: 'SET NULL',
                onUpdate: 'NO ACTION',
                type: attribute_1.DataTypes.STRING(),
            });
            chai_1.expect(joineeModel.aliasOnJoin).to.be.equals('a');
            chai_1.expect(joineeModel.aliasOnTarget).to.be.equals('all_a');
            chai_1.expect(joineeModel.key).to.be.equals('someId');
            chai_1.expect(joineeModel.model).to.be.equals(model);
            chai_1.expect(joineeModel.onDelete).to.be.equals('SET NULL');
            chai_1.expect(joineeModel.onUpdate).to.be.equals('NO ACTION');
            chai_1.expect(joineeModel.type).to.be.instanceof(attribute_1.StringType);
        });
    });
    describe('createJoinModel', function () {
        it('should properly build an UninitializedStateModel object', function () {
            var modelA = mockModel('A');
            var modelB = mockModel('B');
            var joinModel = joinModel_1.createJoinModel({
                firstJoinee: joinModel_1.joinee(mockUninitializedStateModel(modelA), 'a', 'all_a'),
                name: 'a',
                owner: ['b', 'c'],
                secondJoinee: joinModel_1.joinee(mockUninitializedStateModel(modelB), 'b', 'all_b'),
                virtual: true,
            });
            chai_1.expect(joinModel.join).to.be.true;
            chai_1.expect(joinModel.owner).to.deep.equal(['b', 'c']);
            chai_1.expect(joinModel.name).to.be.equals('a');
            chai_1.expect(joinModel.virtual).to.be.true;
            chai_1.expect(joinModel.initialize).to.be.a('function');
        });
        it('should properly add a unique constraint on the join model', function () {
            var modelA = mockModel('A');
            var modelB = mockModel('B');
            var joinModel = joinModel_1.createJoinModel({
                firstJoinee: joinModel_1.joinee(mockUninitializedStateModel(modelA), 'a', 'all_a'),
                name: 'a',
                owner: ['b', 'c'],
                secondJoinee: joinModel_1.joinee(mockUninitializedStateModel(modelB), 'b', 'all_b'),
                virtual: true,
            });
            chai_1.expect(joinModel.model.indexes.unique[0]).to.deep.equal(['aId', 'bId']);
        });
        it('should properly append relations and attributes to the join model', function () {
            var modelA = mockModel('A');
            var modelB = mockModel('B');
            var joinModel = joinModel_1.createJoinModel({
                firstJoinee: joinModel_1.joinee(mockUninitializedStateModel(modelA), 'a', 'all_a'),
                name: 'a',
                owner: ['b', 'c'],
                secondJoinee: joinModel_1.joinee(mockUninitializedStateModel(modelB), 'b', 'all_b'),
                virtual: true,
            });
            chai_1.expect(joinModel.model.attributes[0].name).to.be.equals('aId');
            chai_1.expect(joinModel.model.attributes[0].type).to.be.instanceof(attribute_1.IntegerType);
            chai_1.expect(joinModel.model.attributes[0].required).to.be.true;
            chai_1.expect(joinModel.model.attributes[1].name).to.be.equals('bId');
            chai_1.expect(joinModel.model.attributes[1].type).to.be.instanceof(attribute_1.IntegerType);
            chai_1.expect(joinModel.model.attributes[1].required).to.be.true;
            chai_1.expect(joinModel.model.relations[0].type).to.be.equals('MASTER');
            chai_1.expect(joinModel.model.relations[0].owner).to.deep.equal(['b', 'c']);
            chai_1.expect(joinModel.model.relations[0].alias).to.be.equals('a');
            chai_1.expect(joinModel.model.relations[0].key).to.be.equals('aId');
            chai_1.expect(joinModel.model.relations[0].model).to.be.equals('A');
            chai_1.expect(joinModel.model.relations[0].onDelete).to.be.equals('CASCADE');
            chai_1.expect(joinModel.model.relations[0].onUpdate).to.be.equals('CASCADE');
            chai_1.expect(joinModel.model.relations[0].required).to.be.true;
            chai_1.expect(joinModel.model.relations[0].virtual).to.be.false;
            chai_1.expect(joinModel.model.relations[1].type).to.be.equals('MASTER');
            chai_1.expect(joinModel.model.relations[1].owner).to.deep.equal(['b', 'c']);
            chai_1.expect(joinModel.model.relations[1].alias).to.be.equals('b');
            chai_1.expect(joinModel.model.relations[1].key).to.be.equals('bId');
            chai_1.expect(joinModel.model.relations[1].model).to.be.equals('B');
            chai_1.expect(joinModel.model.relations[1].onDelete).to.be.equals('CASCADE');
            chai_1.expect(joinModel.model.relations[1].onUpdate).to.be.equals('CASCADE');
            chai_1.expect(joinModel.model.relations[1].required).to.be.true;
            chai_1.expect(joinModel.model.relations[1].virtual).to.be.false;
        });
        it('should properly append relations to the joined models', function () {
            var modelA = mockModel('A');
            var modelB = mockModel('B');
            joinModel_1.createJoinModel({
                firstJoinee: joinModel_1.joinee(mockUninitializedStateModel(modelA), 'a', 'all_a'),
                name: 'J',
                owner: ['b', 'c'],
                secondJoinee: joinModel_1.joinee(mockUninitializedStateModel(modelB), 'b', 'all_b'),
                virtual: true,
            });
            chai_1.expect(modelA.relations[0].type).to.be.equals('MANY_TO_MANY');
            chai_1.expect(modelA.relations[0].owner).to.deep.equal(['b', 'c']);
            chai_1.expect(modelA.relations[0].alias).to.be.equals('all_b');
            chai_1.expect(modelA.relations[0].key).to.be.equals('aId');
            chai_1.expect(modelA.relations[0].model).to.be.equals('B');
            chai_1.expect(modelA.relations[0].through).to.be.equals('J');
            chai_1.expect(modelA.relations[0].onDelete).to.be.equals('CASCADE');
            chai_1.expect(modelA.relations[0].onUpdate).to.be.equals('CASCADE');
            chai_1.expect(modelA.relations[0].required).to.be.true;
            chai_1.expect(modelA.relations[0].virtual).to.be.true;
            chai_1.expect(modelB.relations[0].type).to.be.equals('MANY_TO_MANY');
            chai_1.expect(modelB.relations[0].owner).to.deep.equal(['b', 'c']);
            chai_1.expect(modelB.relations[0].alias).to.be.equals('all_a');
            chai_1.expect(modelB.relations[0].key).to.be.equals('bId');
            chai_1.expect(modelB.relations[0].model).to.be.equals('A');
            chai_1.expect(modelB.relations[0].through).to.be.equals('J');
            chai_1.expect(modelB.relations[0].onDelete).to.be.equals('CASCADE');
            chai_1.expect(modelB.relations[0].onUpdate).to.be.equals('CASCADE');
            chai_1.expect(modelB.relations[0].required).to.be.true;
            chai_1.expect(modelB.relations[0].virtual).to.be.true;
        });
        it('should properly add other attributes to the join model', function () {
            var modelA = mockModel('A');
            var modelB = mockModel('B');
            var joinModel = joinModel_1.createJoinModel({
                firstJoinee: joinModel_1.joinee(mockUninitializedStateModel(modelA), 'a', 'all_a'),
                name: 'a',
                owner: ['b', 'c'],
                secondJoinee: joinModel_1.joinee(mockUninitializedStateModel(modelB), 'b', 'all_b'),
                virtual: true,
            }, function (_a) {
                var attribute = _a.attribute;
                attribute({
                    name: 'count',
                    type: attribute_1.DataTypes.INTEGER(),
                });
            });
            joinModel.initialize();
            chai_1.expect(joinModel.model.attributes[2].name).to.be.equals('count');
        });
    });
});
