"use strict";
var lodash_1 = require('lodash');
var attribute_1 = require('./attribute');
function appendRelation(model) {
    var relations = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        relations[_i - 1] = arguments[_i];
    }
    var relation = lodash_1.assign.apply(void 0, [{}].concat(relations));
    if (process.env.NODE_ENV !== 'production') {
        if (!model) {
            throw new Error('Model must be provided.');
        }
        if (relations.length === 0) {
            throw new Error("No relation provided to be appended to " + model.name + ".");
        }
        if (!relation.alias) {
            throw new Error('Relation should have an alias.');
        }
        if (!relation.key) {
            throw new Error('Relation should have a key.');
        }
        if (model.relations.filter(function (r) { return r.alias === relation.alias; }).length > 0) {
            throw new Error(("relation from " + model.name + " to " + relation.model + " as " + relation.alias + " ") +
                "cannot be applied because there is already another relation with the same alias.");
        }
        switch (relation.type) {
            case 'SUPERCLASS':
            case 'SUBCLASS':
            case 'MASTER':
            case 'DETAIL':
            case 'RECURSIVE':
            case 'MANY_TO_MANY':
            case 'INJECTIVE':
                break;
            default:
                throw new Error((relation.type + " is not a valid relation type. ") +
                    ("Please check " + model.name + "." + relation.alias + "."));
        }
        if (relation.type === 'MANY_TO_MANY' && !relation.through) {
            throw new Error("Many to many relations need a join table. " +
                ("Please check " + model.name + "." + relation.alias));
        }
        if (relation.type === 'RECURSIVE' && relation.model !== model.name) {
            throw new Error("Recursive relations must reference their own model. " +
                ("Please check " + model.name + "." + relation.alias));
        }
        var checkMod = function checkMod(mod) {
            switch (mod) {
                case 'RESTRICT':
                case 'CASCADE':
                case 'NO ACTION':
                case 'SET NULL':
                case 'SET DEFAULT':
                    break;
                default:
                    throw new Error((mod + " is not a valid modification action type. ") +
                        ("Please check " + model.name + "." + relation.alias + "."));
            }
        };
        checkMod(relation.onDelete);
        checkMod(relation.onUpdate);
    }
    model.relations.push(relation);
}
exports.appendRelation = appendRelation;
function inheritors(model, options) {
    var typeEnumValues = [];
    options.inheritors.forEach(function (inheritor) {
        appendRelation(model, {
            alias: inheritor.alias,
            key: options.sharedExistingUniqueField,
            model: inheritor.target.name,
            onDelete: 'CASCADE',
            onUpdate: 'CASCADE',
            owner: model.owner,
            required: false,
            type: 'SUBCLASS',
            virtual: inheritor.target.virtual,
        });
        appendRelation(inheritor.target.model, {
            alias: options.aliasOnSubclasses,
            key: options.sharedExistingUniqueField,
            model: model.name,
            onDelete: 'CASCADE',
            onUpdate: 'CASCADE',
            owner: model.owner,
            required: false,
            type: 'SUPERCLASS',
            virtual: model.virtual,
        });
        typeEnumValues.push({ i18nKey: inheritor.i18nKey, name: inheritor.alias });
    });
    var typeEnum = attribute_1.DataTypes.ENUM(options.typeIdentifierEnumTypeName || model.name + 'SubclassType', typeEnumValues, typeEnumValues.map(function (t) { return ("'" + t.name + "'"); }).join(' | '));
    attribute_1.appendAttribute(model, {
        mutable: true,
        name: options.typeIdentifierFieldName || 'type',
        required: !!options.typeIdentifierRequired,
        type: typeEnum,
        validator: options.typeIdentifierValidator,
    });
}
function master(model, options) {
    appendRelation(model, {
        alias: options.alias,
        key: options.foreignKey || options.alias + "Id",
        model: options.target.name,
        onDelete: options.onDelete || 'RESTRICT',
        onUpdate: options.onUpdate || 'CASCADE',
        owner: options.owner || model.owner,
        required: options.required || false,
        type: 'MASTER',
        virtual: options.target.virtual,
    });
    appendRelation(options.target.model, {
        alias: options.ownAliasOnTarget,
        key: options.foreignKey || options.alias + "Id",
        model: model.name,
        onDelete: options.onDelete || 'RESTRICT',
        onUpdate: options.onUpdate || 'CASCADE',
        owner: options.targetOwner || model.owner,
        required: options.required || false,
        type: 'DETAIL',
        virtual: model.virtual,
    });
    attribute_1.appendAttribute(model, {
        name: options.foreignKey || options.alias + "Id",
        required: options.required || false,
        type: options.foreignKeyType || attribute_1.DataTypes.INTEGER(),
        validator: options.validator,
    });
}
function injective(model, options) {
    appendRelation(model, {
        alias: options.alias,
        key: options.foreignKey || options.alias + "Id",
        model: options.target.name,
        onDelete: options.onDelete || 'CASCADE',
        onUpdate: options.onUpdate || 'CASCADE',
        owner: options.owner || model.owner,
        required: options.required || false,
        type: 'MASTER',
        virtual: options.target.virtual,
    });
    appendRelation(options.target.model, {
        alias: options.ownAliasOnTarget,
        key: options.foreignKey || options.alias + "Id",
        model: model.name,
        onDelete: options.onDelete || 'CASCADE',
        onUpdate: options.onUpdate || 'CASCADE',
        owner: options.targetOwner || model.owner,
        required: options.required || false,
        type: 'INJECTIVE',
        virtual: model.virtual,
    });
    attribute_1.appendAttribute(model, {
        name: options.foreignKey || options.alias + "Id",
        required: options.required || false,
        type: options.foreignKeyType || attribute_1.DataTypes.INTEGER(),
        validator: options.validator,
    });
}
function recursive(model, options) {
    appendRelation(model, {
        alias: options.alias,
        key: options.key || options.alias + "Id",
        model: model.name,
        onDelete: options.onDelete || 'RESTRICT',
        onUpdate: options.onUpdate || 'CASCADE',
        owner: model.owner,
        required: false,
        type: 'RECURSIVE',
        virtual: model.virtual,
    });
    attribute_1.appendAttribute(model, {
        name: options.key || options.alias + "Id",
        required: false,
        type: options.keyType || attribute_1.DataTypes.INTEGER(),
        validator: options.validator,
    });
}
function createRelationBuilder(model) {
    return {
        inheritors: function (options) { return inheritors(model, options); },
        injective: function (options) { return injective(model, options); },
        master: function (options) { return master(model, options); },
        recursive: function (options) { return recursive(model, options); },
    };
}
exports.createRelationBuilder = createRelationBuilder;
