"use strict";
var chai_1 = require('chai');
var attribute_1 = require('./attribute');
var relation_1 = require('./relation');
function mockModel(name) {
    return {
        name: name,
        attributes: [],
        indexes: { unique: [] },
        join: false,
        owner: null,
        relations: [],
        virtual: false,
    };
}
function mockUninitializedStateModel(model) {
    return {
        model: model,
        initialize: function () { return null; },
        join: model.join,
        name: model.name,
        owner: model.owner,
        virtual: model.virtual,
    };
}
var baseRelation = {
    alias: null,
    key: 'aId',
    model: 'A',
    onDelete: 'CASCADE',
    onUpdate: 'CASCADE',
    owner: null,
    type: null,
};
describe('relation', function () {
    describe('appendRelation', function () {
        it('should properly append the relation to the model', function () {
            var model = mockModel('A');
            relation_1.appendRelation(model, baseRelation, { alias: 'a', type: 'RECURSIVE' });
            relation_1.appendRelation(model, baseRelation, { alias: 'b', type: 'RECURSIVE' });
            chai_1.expect(model.relations[0].alias).to.be.equals('a');
            chai_1.expect(model.relations[1].alias).to.be.equals('b');
        });
        it('should fail when model is invalid', function () {
            chai_1.expect(function () { return relation_1.appendRelation(null); }).to.throw();
        });
        it('should fail when no relations are provided', function () {
            chai_1.expect(function () { return relation_1.appendRelation(mockModel('A')); }).to.throw();
        });
        it('should fail when alias is invalid', function () {
            var model = mockModel('A');
            chai_1.expect(function () {
                return relation_1.appendRelation(model, baseRelation, { alias: null, type: 'RECURSIVE' });
            }).to.throw();
        });
        it('should fail when key is invalid', function () {
            var model = mockModel('A');
            chai_1.expect(function () {
                return relation_1.appendRelation(model, baseRelation, { alias: 'a', key: null, type: 'RECURSIVE' });
            }).to.throw();
        });
        it('should fail when alias is repeated', function () {
            var model = mockModel('A');
            chai_1.expect(function () {
                return relation_1.appendRelation(model, baseRelation, { alias: 'a', type: 'RECURSIVE' });
            }).not.to.throw();
            chai_1.expect(function () {
                return relation_1.appendRelation(model, baseRelation, { alias: 'a', type: 'RECURSIVE' });
            }).to.throw();
        });
        it('should fail when type is invalid', function () {
            var model = mockModel('A');
            chai_1.expect(function () {
                return relation_1.appendRelation(model, baseRelation, { alias: 'a', type: 'BLAH' });
            }).to.throw();
        });
        it('should fail when type is many to many but through is not provided', function () {
            var model = mockModel('A');
            chai_1.expect(function () {
                return relation_1.appendRelation(model, baseRelation, { alias: 'a', type: 'MANY_TO_MANY' });
            }).to.throw();
        });
        it('should fail when type is recursive but model is different', function () {
            var model = mockModel('A');
            chai_1.expect(function () {
                return relation_1.appendRelation(model, baseRelation, { alias: 'a', model: 'B', type: 'RECURSIVE' });
            }).to.throw();
        });
        it('should fail when onUpdate or onDelete are invalid', function () {
            var model = mockModel('A');
            chai_1.expect(function () {
                return relation_1.appendRelation(model, baseRelation, { alias: 'a', onDelete: 'A', type: 'RECURSIVE' });
            }).to.throw();
            chai_1.expect(function () {
                return relation_1.appendRelation(model, baseRelation, { alias: 'a', onUpdate: 'A', type: 'RECURSIVE' });
            }).to.throw();
        });
    });
    describe('createRelationBuilder', function () {
        describe('inheritors builder', function () {
            it('should properly initialize an inheritors builder', function () {
                var main = mockModel('Main');
                chai_1.expect(function () {
                    return relation_1.createRelationBuilder(main).inheritors;
                }).not.to.throw();
            });
            function assetEnumType(main, index, typeName, typeEnumName, enumValueNames) {
                chai_1.expect(main.attributes[index].name).to.be.equals(typeName);
                chai_1.expect(main.attributes[index].type).to.be.instanceof(attribute_1.EnumType);
                chai_1.expect(main.attributes[index].type.name)
                    .to.be.equals(typeEnumName);
                chai_1.expect(main.attributes[index].type.typeName)
                    .to.be.equals(enumValueNames.map(function (t) { return ("'" + t + "'"); }).join(' | '));
                chai_1.expect(main.attributes[index].type.values.map(function (v) { return v.name; }))
                    .to.deep.equal(enumValueNames);
            }
            var testCases = [
                [
                    'should add the type enum to the main model',
                    function (main, inheritor, append) {
                        append({
                            aliasOnSubclasses: 'main',
                            inheritors: [
                                inheritor('Child1', 'a', 'A'),
                                inheritor('Child2', 'b', 'A'),
                                inheritor('Child3', 'c', 'A'),
                            ],
                            sharedExistingUniqueField: 'id',
                        });
                        assetEnumType(main, 0, 'type', 'MainSubclassType', ['a', 'b', 'c']);
                    },
                ],
                [
                    'should add the type enum with required and validator',
                    function (main, inheritor, append) {
                        var validator = function () { return null; };
                        append({
                            aliasOnSubclasses: 'main',
                            inheritors: [inheritor('Child1', 'a', 'A')],
                            sharedExistingUniqueField: 'id',
                            typeIdentifierRequired: true,
                            typeIdentifierValidator: validator,
                        });
                        assetEnumType(main, 0, 'type', 'MainSubclassType', ['a']);
                        chai_1.expect(main.attributes[0].required).to.be.true;
                        chai_1.expect(main.attributes[0].validator).to.be.equals(validator);
                    },
                ],
                [
                    'should correctly add the subclass/superclass relation pairs',
                    function (main, inheritor, append) {
                        main.owner = 'owner';
                        main.virtual = true;
                        var child1 = inheritor('Child1', 'a', 'A');
                        var child2 = inheritor('Child2', 'b', 'A');
                        var child3 = inheritor('Child3', 'c', 'A');
                        child3.target.virtual = true;
                        child3.target.model.virtual = true;
                        append({
                            aliasOnSubclasses: 'main',
                            inheritors: [child1, child2, child3],
                            sharedExistingUniqueField: 'id',
                        });
                        chai_1.expect(main.relations[0].owner).to.be.equals('owner');
                        chai_1.expect(main.relations[0].type).to.be.equals('SUBCLASS');
                        chai_1.expect(main.relations[0].model).to.be.equals('Child1');
                        chai_1.expect(main.relations[0].alias).to.be.equals('a');
                        chai_1.expect(main.relations[0].key).to.be.equals('id');
                        chai_1.expect(main.relations[0].virtual).to.be.false;
                        chai_1.expect(child1.target.model.relations[0].owner).to.be.equals('owner');
                        chai_1.expect(child1.target.model.relations[0].type).to.be.equals('SUPERCLASS');
                        chai_1.expect(child1.target.model.relations[0].model).to.be.equals('Main');
                        chai_1.expect(child1.target.model.relations[0].alias).to.be.equals('main');
                        chai_1.expect(child1.target.model.relations[0].key).to.be.equals('id');
                        chai_1.expect(child1.target.model.relations[0].virtual).to.be.true;
                        chai_1.expect(main.relations[1].owner).to.be.equals('owner');
                        chai_1.expect(main.relations[1].type).to.be.equals('SUBCLASS');
                        chai_1.expect(main.relations[1].model).to.be.equals('Child2');
                        chai_1.expect(main.relations[1].alias).to.be.equals('b');
                        chai_1.expect(main.relations[1].key).to.be.equals('id');
                        chai_1.expect(main.relations[1].virtual).to.be.false;
                        chai_1.expect(child2.target.model.relations[0].owner).to.be.equals('owner');
                        chai_1.expect(child2.target.model.relations[0].type).to.be.equals('SUPERCLASS');
                        chai_1.expect(child2.target.model.relations[0].model).to.be.equals('Main');
                        chai_1.expect(child2.target.model.relations[0].alias).to.be.equals('main');
                        chai_1.expect(child2.target.model.relations[0].key).to.be.equals('id');
                        chai_1.expect(child2.target.model.relations[0].virtual).to.be.true;
                        chai_1.expect(main.relations[2].owner).to.be.equals('owner');
                        chai_1.expect(main.relations[2].type).to.be.equals('SUBCLASS');
                        chai_1.expect(main.relations[2].model).to.be.equals('Child3');
                        chai_1.expect(main.relations[2].alias).to.be.equals('c');
                        chai_1.expect(main.relations[2].key).to.be.equals('id');
                        chai_1.expect(main.relations[2].virtual).to.be.true;
                        chai_1.expect(child3.target.model.relations[0].owner).to.be.equals('owner');
                        chai_1.expect(child3.target.model.relations[0].type).to.be.equals('SUPERCLASS');
                        chai_1.expect(child3.target.model.relations[0].model).to.be.equals('Main');
                        chai_1.expect(child3.target.model.relations[0].alias).to.be.equals('main');
                        chai_1.expect(child3.target.model.relations[0].key).to.be.equals('id');
                        chai_1.expect(child3.target.model.relations[0].virtual).to.be.true;
                    },
                ],
            ];
            testCases.forEach(function (_a) {
                var expectation = _a[0], testCase = _a[1];
                var main = mockModel('Main');
                var inheritor = function (name, alias, i18nKey) { return ({
                    alias: alias,
                    i18nKey: i18nKey,
                    target: mockUninitializedStateModel(mockModel(name)),
                }); };
                var appendInheritorGroup = relation_1.createRelationBuilder(main).inheritors;
                it(expectation, function () {
                    testCase(main, inheritor, appendInheritorGroup);
                });
            });
        });
        describe('master builder', function () {
            it('should properly initialize a master builder', function () {
                var main = mockModel('Main');
                chai_1.expect(function () {
                    return relation_1.createRelationBuilder(main).inheritors;
                }).not.to.throw();
            });
            it('should properly append master/detail relations and fk on source', function () {
                var master = mockModel('Master');
                var detail = mockModel('Detail');
                relation_1.createRelationBuilder(detail).master({
                    alias: 'master',
                    ownAliasOnTarget: 'details',
                    target: mockUninitializedStateModel(master),
                });
                chai_1.expect(master.relations[0].alias).to.be.equals('details');
                chai_1.expect(master.relations[0].type).to.be.equals('DETAIL');
                chai_1.expect(detail.relations[0].alias).to.be.equals('master');
                chai_1.expect(detail.relations[0].type).to.be.equals('MASTER');
                chai_1.expect(detail.attributes[0].name).to.be.equals('masterId');
                chai_1.expect(detail.attributes[0].type).to.be.instanceof(attribute_1.IntegerType);
            });
            it('should properly override the properties of relations and the fk attribute', function () {
                var master = mockModel('Master');
                var detail = mockModel('Detail');
                var noop = function () { return null; };
                detail.virtual = true;
                relation_1.createRelationBuilder(detail).master({
                    alias: 'master',
                    foreignKey: 'someId',
                    foreignKeyType: attribute_1.DataTypes.STRING(),
                    onDelete: 'RESTRICT',
                    onUpdate: 'NO ACTION',
                    ownAliasOnTarget: 'details',
                    owner: 'owner',
                    required: true,
                    target: mockUninitializedStateModel(master),
                    targetOwner: 'owner2',
                    validator: noop,
                });
                chai_1.expect(master.relations[0].owner).to.be.equals('owner2');
                chai_1.expect(master.relations[0].type).to.be.equals('DETAIL');
                chai_1.expect(master.relations[0].model).to.be.equals('Detail');
                chai_1.expect(master.relations[0].alias).to.be.equals('details');
                chai_1.expect(master.relations[0].key).to.be.equals('someId');
                chai_1.expect(master.relations[0].required).to.be.equals(true);
                chai_1.expect(master.relations[0].virtual).to.be.equals(true);
                chai_1.expect(master.relations[0].onDelete).to.be.equals('RESTRICT');
                chai_1.expect(master.relations[0].onUpdate).to.be.equals('NO ACTION');
                chai_1.expect(detail.relations[0].owner).to.be.equals('owner');
                chai_1.expect(detail.relations[0].type).to.be.equals('MASTER');
                chai_1.expect(detail.relations[0].model).to.be.equals('Master');
                chai_1.expect(detail.relations[0].alias).to.be.equals('master');
                chai_1.expect(detail.relations[0].key).to.be.equals('someId');
                chai_1.expect(detail.relations[0].required).to.be.equals(true);
                chai_1.expect(detail.relations[0].virtual).to.be.equals(false);
                chai_1.expect(detail.relations[0].onDelete).to.be.equals('RESTRICT');
                chai_1.expect(detail.relations[0].onUpdate).to.be.equals('NO ACTION');
                chai_1.expect(detail.attributes[0].name).to.be.equals('someId');
                chai_1.expect(detail.attributes[0].type).to.be.instanceof(attribute_1.StringType);
                chai_1.expect(detail.attributes[0].required).to.be.equals(true);
                chai_1.expect(detail.attributes[0].validator).to.be.equals(noop);
            });
        });
        describe('injective builder', function () {
            it('should properly initialize a injective builder', function () {
                var main = mockModel('Main');
                chai_1.expect(function () {
                    return relation_1.createRelationBuilder(main).inheritors;
                }).not.to.throw();
            });
            it('should properly append injective/master relations and fk on source', function () {
                var master = mockModel('Master');
                var injected = mockModel('Injected');
                relation_1.createRelationBuilder(injected).injective({
                    alias: 'master',
                    ownAliasOnTarget: 'injected',
                    target: mockUninitializedStateModel(master),
                });
                chai_1.expect(master.relations[0].alias).to.be.equals('injected');
                chai_1.expect(master.relations[0].type).to.be.equals('INJECTIVE');
                chai_1.expect(injected.relations[0].alias).to.be.equals('master');
                chai_1.expect(injected.relations[0].type).to.be.equals('MASTER');
                chai_1.expect(injected.attributes[0].name).to.be.equals('masterId');
                chai_1.expect(injected.attributes[0].type).to.be.instanceof(attribute_1.IntegerType);
            });
            it('should properly override the properties of relations and the fk attribute', function () {
                var master = mockModel('Master');
                var injected = mockModel('Injected');
                var noop = function () { return null; };
                injected.virtual = true;
                relation_1.createRelationBuilder(injected).injective({
                    alias: 'master',
                    foreignKey: 'someId',
                    foreignKeyType: attribute_1.DataTypes.STRING(),
                    onDelete: 'RESTRICT',
                    onUpdate: 'NO ACTION',
                    ownAliasOnTarget: 'injected',
                    owner: 'owner',
                    required: true,
                    target: mockUninitializedStateModel(master),
                    targetOwner: 'owner2',
                    validator: noop,
                });
                chai_1.expect(master.relations[0].owner).to.be.equals('owner2');
                chai_1.expect(master.relations[0].type).to.be.equals('INJECTIVE');
                chai_1.expect(master.relations[0].model).to.be.equals('Injected');
                chai_1.expect(master.relations[0].alias).to.be.equals('injected');
                chai_1.expect(master.relations[0].key).to.be.equals('someId');
                chai_1.expect(master.relations[0].required).to.be.equals(true);
                chai_1.expect(master.relations[0].virtual).to.be.equals(true);
                chai_1.expect(master.relations[0].onDelete).to.be.equals('RESTRICT');
                chai_1.expect(master.relations[0].onUpdate).to.be.equals('NO ACTION');
                chai_1.expect(injected.relations[0].owner).to.be.equals('owner');
                chai_1.expect(injected.relations[0].type).to.be.equals('MASTER');
                chai_1.expect(injected.relations[0].model).to.be.equals('Master');
                chai_1.expect(injected.relations[0].alias).to.be.equals('master');
                chai_1.expect(injected.relations[0].key).to.be.equals('someId');
                chai_1.expect(injected.relations[0].required).to.be.equals(true);
                chai_1.expect(injected.relations[0].virtual).to.be.equals(false);
                chai_1.expect(injected.relations[0].onDelete).to.be.equals('RESTRICT');
                chai_1.expect(injected.relations[0].onUpdate).to.be.equals('NO ACTION');
                chai_1.expect(injected.attributes[0].name).to.be.equals('someId');
                chai_1.expect(injected.attributes[0].type).to.be.instanceof(attribute_1.StringType);
                chai_1.expect(injected.attributes[0].required).to.be.equals(true);
                chai_1.expect(injected.attributes[0].validator).to.be.equals(noop);
            });
        });
        describe('recursive builder', function () {
            it('should properly initialize a recursive builder', function () {
                var main = mockModel('Main');
                chai_1.expect(function () {
                    return relation_1.createRelationBuilder(main).recursive;
                }).not.to.throw();
            });
            it('should properly append the recursive relation and its key', function () {
                var master = mockModel('Master');
                relation_1.createRelationBuilder(master).recursive({
                    alias: 'parent',
                });
                chai_1.expect(master.relations[0].alias).to.be.equals('parent');
                chai_1.expect(master.relations[0].type).to.be.equals('RECURSIVE');
                chai_1.expect(master.attributes[0].name).to.be.equals('parentId');
                chai_1.expect(master.attributes[0].type).to.be.instanceof(attribute_1.IntegerType);
            });
            it('should properly override the properties of the relation and the key', function () {
                var master = mockModel('Master');
                var noop = function () { return null; };
                master.owner = 'owner';
                master.virtual = true;
                relation_1.createRelationBuilder(master).recursive({
                    alias: 'parent',
                    key: 'someId',
                    keyType: attribute_1.DataTypes.STRING(),
                    onDelete: 'RESTRICT',
                    onUpdate: 'NO ACTION',
                    validator: noop,
                });
                chai_1.expect(master.relations[0].owner).to.be.equals('owner');
                chai_1.expect(master.relations[0].type).to.be.equals('RECURSIVE');
                chai_1.expect(master.relations[0].model).to.be.equals('Master');
                chai_1.expect(master.relations[0].alias).to.be.equals('parent');
                chai_1.expect(master.relations[0].key).to.be.equals('someId');
                chai_1.expect(master.relations[0].required).to.be.equals(false);
                chai_1.expect(master.relations[0].virtual).to.be.equals(true);
                chai_1.expect(master.relations[0].onDelete).to.be.equals('RESTRICT');
                chai_1.expect(master.relations[0].onUpdate).to.be.equals('NO ACTION');
                chai_1.expect(master.attributes[0].name).to.be.equals('someId');
                chai_1.expect(master.attributes[0].type).to.be.instanceof(attribute_1.StringType);
                chai_1.expect(master.attributes[0].required).to.be.equals(false);
                chai_1.expect(master.attributes[0].validator).to.be.equals(noop);
            });
        });
    });
});
