"use strict";
var chai_1 = require('chai');
var spec_1 = require('./spec');
function mockModel(name) {
    return {
        name: name,
        attributes: [],
        indexes: { unique: [] },
        join: false,
        owner: null,
        relations: [],
        validator: null,
        virtual: false,
    };
}
describe('spec', function () {
    it('should sucessfully instantiate a Spec object', function () {
        chai_1.expect(function () { return new spec_1.Spec(mockModel('a')); }).not.to.throw();
    });
    it('should properly forward the model properties without caching', function () {
        var model = mockModel('a');
        var spec = new spec_1.Spec(model);
        var noop = function () { return null; };
        chai_1.expect(spec.name).to.be.equals('a');
        chai_1.expect(spec.owner).to.be.equals(null);
        chai_1.expect(spec.virtual).to.be.equals(false);
        chai_1.expect(spec.join).to.be.equals(false);
        chai_1.expect(spec.validator).to.be.equals(null);
        model.name = 'c';
        model.owner = 'b';
        model.virtual = true;
        model.join = true;
        model.validator = noop;
        chai_1.expect(spec.name).to.be.equals('c');
        chai_1.expect(spec.owner).to.be.equals('b');
        chai_1.expect(spec.virtual).to.be.equals(true);
        chai_1.expect(spec.join).to.be.equals(true);
        chai_1.expect(spec.validator).to.be.equals(noop);
        chai_1.expect(spec.attributeArray).to.be.equals(model.attributes);
        chai_1.expect(spec.indexes).to.be.equals(model.indexes);
        chai_1.expect(spec.relationArray).to.be.equals(model.relations);
    });
    it('should properly compute the attribute map', function () {
        var model = mockModel('a');
        var spec = new spec_1.Spec(model);
        chai_1.expect(Object.keys(spec.attributes).length).to.be.equals(0);
        model.attributes.push({ name: 'id', owner: null, type: null });
        chai_1.expect(Object.keys(spec.attributes).length).to.be.equals(1);
        chai_1.expect(spec.attributes['id'].name).to.be.equals('id');
    });
    it('should properly compute the relations object', function () {
        var model = mockModel('a');
        var spec = new spec_1.Spec(model);
        chai_1.expect(Object.keys(spec.relations.superclass).length).to.be.equals(0);
        chai_1.expect(Object.keys(spec.relations.subclass).length).to.be.equals(0);
        chai_1.expect(Object.keys(spec.relations.master).length).to.be.equals(0);
        chai_1.expect(Object.keys(spec.relations.detail).length).to.be.equals(0);
        chai_1.expect(Object.keys(spec.relations.recursive).length).to.be.equals(0);
        chai_1.expect(Object.keys(spec.relations.manyToMany).length).to.be.equals(0);
        chai_1.expect(Object.keys(spec.relations.injective).length).to.be.equals(0);
        model.relations.push({ alias: 'a', model: 'A', type: 'SUPERCLASS' });
        model.relations.push({ alias: 'a', model: 'A', type: 'SUBCLASS' });
        model.relations.push({ alias: 'a', model: 'A', type: 'MASTER' });
        model.relations.push({ alias: 'a', model: 'A', type: 'DETAIL' });
        model.relations.push({ alias: 'a', model: 'A', type: 'RECURSIVE' });
        model.relations.push({ alias: 'a', model: 'A', type: 'MANY_TO_MANY' });
        model.relations.push({ alias: 'a', model: 'A', type: 'INJECTIVE' });
        model.relations.push({ alias: 'a', model: 'A', type: 'SUBCLASS' });
        model.relations.push({ alias: 'a', model: 'A', type: 'MASTER' });
        model.relations.push({ alias: 'a', model: 'A', type: 'RECURSIVE' });
        model.relations.push({ alias: 'hello', model: 'H', type: 'INJECTIVE' });
        chai_1.expect(Object.keys(spec.relations.superclass).length).to.be.equals(1);
        chai_1.expect(Object.keys(spec.relations.subclass).length).to.be.equals(2);
        chai_1.expect(Object.keys(spec.relations.master).length).to.be.equals(2);
        chai_1.expect(Object.keys(spec.relations.detail).length).to.be.equals(1);
        chai_1.expect(Object.keys(spec.relations.recursive).length).to.be.equals(2);
        chai_1.expect(Object.keys(spec.relations.manyToMany).length).to.be.equals(1);
        chai_1.expect(Object.keys(spec.relations.injective).length).to.be.equals(2);
        chai_1.expect(spec.relations.byAlias('hello').model).to.be.equals('H');
    });
    it('should properly compute the non virtual relations object', function () {
        var model = mockModel('a');
        var spec = new spec_1.Spec(model);
        chai_1.expect(Object.keys(spec.nonVirtualRelations.superclass).length).to.be.equals(0);
        chai_1.expect(Object.keys(spec.nonVirtualRelations.subclass).length).to.be.equals(0);
        chai_1.expect(Object.keys(spec.nonVirtualRelations.master).length).to.be.equals(0);
        chai_1.expect(Object.keys(spec.nonVirtualRelations.detail).length).to.be.equals(0);
        chai_1.expect(Object.keys(spec.nonVirtualRelations.recursive).length).to.be.equals(0);
        chai_1.expect(Object.keys(spec.nonVirtualRelations.manyToMany).length).to.be.equals(0);
        chai_1.expect(Object.keys(spec.nonVirtualRelations.injective).length).to.be.equals(0);
        model.relations.push({ alias: 'a', model: 'A', type: 'SUPERCLASS', virtual: false });
        model.relations.push({ alias: 'a', model: 'A', type: 'SUBCLASS', virtual: true });
        model.relations.push({ alias: 'a', model: 'A', type: 'MASTER', virtual: true });
        model.relations.push({ alias: 'a', model: 'A', type: 'DETAIL', virtual: false });
        model.relations.push({ alias: 'a', model: 'A', type: 'RECURSIVE', virtual: true });
        model.relations.push({ alias: 'a', model: 'A', type: 'MANY_TO_MANY', virtual: true });
        model.relations.push({ alias: 'a', model: 'A', type: 'INJECTIVE', virtual: false });
        model.relations.push({ alias: 'a', model: 'A', type: 'SUBCLASS', virtual: false });
        model.relations.push({ alias: 'a', model: 'A', type: 'MASTER', virtual: true });
        model.relations.push({ alias: 'a', model: 'A', type: 'RECURSIVE', virtual: false });
        model.relations.push({ alias: 'hello', model: 'H', type: 'INJECTIVE', virtual: false });
        chai_1.expect(Object.keys(spec.nonVirtualRelations.superclass).length).to.be.equals(1);
        chai_1.expect(Object.keys(spec.nonVirtualRelations.subclass).length).to.be.equals(1);
        chai_1.expect(Object.keys(spec.nonVirtualRelations.master).length).to.be.equals(0);
        chai_1.expect(Object.keys(spec.nonVirtualRelations.detail).length).to.be.equals(1);
        chai_1.expect(Object.keys(spec.nonVirtualRelations.recursive).length).to.be.equals(1);
        chai_1.expect(Object.keys(spec.nonVirtualRelations.manyToMany).length).to.be.equals(0);
        chai_1.expect(Object.keys(spec.nonVirtualRelations.injective).length).to.be.equals(2);
        chai_1.expect(spec.nonVirtualRelations.byAlias('hello').model).to.be.equals('H');
        model.relations.push({ alias: 'world', model: 'W', type: 'INJECTIVE', virtual: true });
        chai_1.expect(spec.nonVirtualRelations.byAlias('world')).to.be.undefined;
    });
    it('should properly compute the dependencies', function () {
        var model = mockModel('a');
        var spec = new spec_1.Spec(model);
        chai_1.expect(spec.dependencies.length).to.be.equals(0);
        model.relations.push({ alias: 'a', model: 'C', type: 'INJECTIVE' });
        model.relations.push({ alias: 'a', model: 'A', through: 'B', type: 'MANY_TO_MANY' });
        chai_1.expect(spec.dependencies).to.deep.equal(['A', 'B', 'C']);
    });
    it('should properly compute the non virtual dependencies', function () {
        var model = mockModel('a');
        var spec = new spec_1.Spec(model);
        chai_1.expect(spec.nonVirtualDependencies.length).to.be.equals(0);
        model.relations.push({ alias: 'a', model: 'C', type: 'INJECTIVE', virtual: true });
        model.relations.push({ alias: 'a', model: 'A', through: 'B', type: 'MANY_TO_MANY' });
        chai_1.expect(spec.nonVirtualDependencies).to.deep.equal(['A', 'B']);
    });
    describe('filterByOwner', function () {
        it('should return the same model if the input or the model owner are not specified', function () {
            var model = mockModel('M');
            var spec = new spec_1.Spec(model);
            chai_1.expect(spec.filterByOwner('a')).to.be.equals(spec);
            chai_1.expect(spec.filterByOwner(['a', 'b'])).to.be.equals(spec);
            model.owner = 'owner';
            chai_1.expect(spec.filterByOwner(null)).to.be.equals(spec);
            chai_1.expect(spec.filterByOwner([])).to.be.equals(spec);
        });
        it('should return null if input and the model owner(s) have non in common', function () {
            var model = mockModel('M');
            var spec = new spec_1.Spec(model);
            model.owner = 'owner';
            chai_1.expect(spec.filterByOwner('a')).to.be.null;
            chai_1.expect(spec.filterByOwner(['a', 'b'])).to.be.null;
        });
        it('should return a new spec if there are intersecting owners', function () {
            var model = mockModel('M');
            var spec = new spec_1.Spec(model);
            model.owner = 'b';
            chai_1.expect(spec.filterByOwner(['a', 'b'])).not.to.be.null;
            chai_1.expect(spec.filterByOwner(['a', 'b'])).not.to.be.equals(spec);
            model.owner = ['b', 'c'];
            chai_1.expect(spec.filterByOwner(['a', 'd'])).to.be.null;
            chai_1.expect(spec.filterByOwner('a')).to.be.null;
            chai_1.expect(spec.filterByOwner(['c', 'b'])).not.to.be.null;
            chai_1.expect(spec.filterByOwner(['c', 'b'])).not.to.be.equals(spec);
            chai_1.expect(spec.filterByOwner('c')).not.to.be.null;
            chai_1.expect(spec.filterByOwner('c')).not.to.be.equals(spec);
        });
        it('should properly filter relations', function () {
            var model = mockModel('M');
            var spec = new spec_1.Spec(model);
            model.owner = 'b';
            model.relations.push({ alias: '1' });
            model.relations.push({ alias: '2', owner: 'c' });
            model.relations.push({ alias: '3', owner: ['c', 'b'] });
            model.relations.push({ alias: '4', owner: null });
            chai_1.expect(spec.filterByOwner('b').relations.byAlias('1')).not.to.be.undefined;
            chai_1.expect(spec.filterByOwner('b').relations.byAlias('2')).to.be.undefined;
            chai_1.expect(spec.filterByOwner('b').relations.byAlias('3')).not.to.be.undefined;
            chai_1.expect(spec.filterByOwner('b').relations.byAlias('4')).not.to.be.undefined;
            chai_1.expect(spec.filterByOwner(['b', 'c']).relations.byAlias('1')).not.to.be.undefined;
            chai_1.expect(spec.filterByOwner(['b', 'c']).relations.byAlias('2')).not.to.be.undefined;
            chai_1.expect(spec.filterByOwner(['b', 'c']).relations.byAlias('3')).not.to.be.undefined;
            chai_1.expect(spec.filterByOwner(['b', 'c']).relations.byAlias('4')).not.to.be.undefined;
        });
        it('should properly filter attributes', function () {
            var model = mockModel('M');
            var spec = new spec_1.Spec(model);
            model.owner = 'b';
            model.attributes.push({ name: '1' });
            model.attributes.push({ name: '2', owner: 'c' });
            model.attributes.push({ name: '3', owner: ['c', 'b'] });
            model.attributes.push({ name: '4', owner: null });
            chai_1.expect(spec.filterByOwner('b').attributes['1']).not.to.be.undefined;
            chai_1.expect(spec.filterByOwner('b').attributes['2']).to.be.undefined;
            chai_1.expect(spec.filterByOwner('b').attributes['3']).not.to.be.undefined;
            chai_1.expect(spec.filterByOwner('b').attributes['4']).not.to.be.undefined;
            chai_1.expect(spec.filterByOwner(['b', 'c']).attributes['1']).not.to.be.undefined;
            chai_1.expect(spec.filterByOwner(['b', 'c']).attributes['2']).not.to.be.undefined;
            chai_1.expect(spec.filterByOwner(['b', 'c']).attributes['3']).not.to.be.undefined;
            chai_1.expect(spec.filterByOwner(['b', 'c']).attributes['4']).not.to.be.undefined;
        });
    });
});
