"use strict";
var Bluebird = require('bluebird');
var fetch_1 = require('./fetch');
function buildSynchronizerCreator(endPointUrl) {
    function buildSynchronizer() {
        var id = 0;
        var synched = false;
        var items = [];
        var listeners = [];
        function addIdAction(item, action) {
            id += 1;
            item.itemId = id;
            item.action = action;
            return item;
        }
        function listen() {
            return new Bluebird(function (resolve, reject) {
                listeners.push({ resolve: resolve, reject: reject });
            });
        }
        function addInsert(insert) {
            var item = addIdAction(insert, 'INSERT');
            return new Bluebird(function (resolve, reject) {
                items.push({ item: item, resolve: resolve, reject: reject });
            });
        }
        function addUpdate(update) {
            var item = addIdAction(update, 'UPDATE');
            return new Bluebird(function (resolve, reject) {
                items.push({ item: item, resolve: resolve, reject: reject });
            });
        }
        function addRemove(remove) {
            var item = addIdAction(remove, 'REMOVE');
            return new Bluebird(function (resolve, reject) {
                items.push({ item: item, resolve: resolve, reject: reject });
            });
        }
        function sync() {
            if (process.env.NODE_ENV !== 'production' && synched) {
                throw new Error('sync cannot be called on the same object more than once');
            }
            if (items.length === 0) {
                return Bluebird.resolve(false);
            }
            synched = true;
            return fetch_1.fetch({
                body: items.map(function (i) { return i.item; }),
                method: 'POST',
                url: endPointUrl,
            }).then(function (response) {
                if (Array.isArray(response.body) && response.body.length > 0) {
                    items.forEach(function (item) {
                        var responseItem = response.body.filter(function (e) { return e.itemId === item.item.itemId; })[0];
                        item.resolve({
                            error: responseItem.error,
                            ok: response.ok,
                            status: response.status,
                            statusText: response.statusText,
                            warning: responseItem.warning,
                        });
                    });
                    var errors_1 = response.body.filter(function (item) { return item.error; }).map(function (item) { return item.error; });
                    var warnings_1 = response.body.filter(function (item) { return item.warning; }).map(function (item) { return item.warning; });
                    listeners.forEach(function (listener) { return listener.resolve({
                        errors: errors_1,
                        warnings: warnings_1,
                        ok: response.ok,
                        status: response.status,
                        statusText: response.statusText,
                    }); });
                }
                else {
                    items.forEach(function (item) { return item.reject(new Error('Invalid response')); });
                    listeners.forEach(function (listener) { return listener.reject(new Error('Invalid response')); });
                }
            }).catch(function (error) {
                listeners.forEach(function (listener) { return listener.reject(error); });
                items.forEach(function (item) { return item.reject(error); });
            }).then(function () { return true; });
        }
        return { listen: listen, addInsert: addInsert, addUpdate: addUpdate, addRemove: addRemove, sync: sync };
    }
    function buildSingleShotSynchronizer(harness) {
        var synchronizer = buildSynchronizer();
        harness(synchronizer);
        return synchronizer.sync();
    }
    return { buildSynchronizer: buildSynchronizer, buildSingleShotSynchronizer: buildSingleShotSynchronizer };
}
exports.buildSynchronizerCreator = buildSynchronizerCreator;
